#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is a test for writing the SigNetSim data structure into a NuML document

"""

from libsignetsim import Settings, Experiment

from unittest import TestCase
from os.path import join


class TestReadWriteData(TestCase):
	""" Tests high level functions """

	def testReadWriteData(self):

		# Initializing data structure
		experiment = Experiment("Ras-GTP, ERK1-2 quantifications")
		experiment.notes = "Elisa quantifications for RAS-GTP, Western blot quanfitication for ERK1-2"
		condition_0 = experiment.createCondition("Starved")

		condition_0.addInitialCondition(0, "FGF2", 0)
		condition_0.notes = "starved cells for 48h"
		condition_0.addObservation(0, "Total Ras-GTP", 40)
		condition_0.addObservation(60, "Total Ras-GTP", 40)
		condition_0.addObservation(180, "Total Ras-GTP", 40)
		condition_0.addObservation(300, "Total Ras-GTP", 40)
		condition_0.addObservation(900, "Total Ras-GTP", 40)
		condition_0.addObservation(1800, "Total Ras-GTP", 40)
		condition_0.addObservation(3600, "Total Ras-GTP", 40)

		condition_1 = experiment.createCondition("FGF2")
		condition_1.notes = "FGF2 treatment"
		condition_1.addInitialCondition(0, "FGF2", 333)

		condition_1.addObservation(0, "Total Ras-GTP", 40)
		condition_1.addObservation(60, "Total Ras-GTP", 92)
		condition_1.addObservation(180, "Total Ras-GTP", 77)
		condition_1.addObservation(300, "Total Ras-GTP", 333)
		condition_1.addObservation(900, "Total Ras-GTP", 222)
		condition_1.addObservation(1800, "Total Ras-GTP", 68)
		condition_1.addObservation(3600, "Total Ras-GTP", 60)

		experiment.writeNuMLToFile(join(Settings.tempDirectory, "data.xml"))

		experiment_imported = Experiment()
		experiment_imported.readNuMLFromFile(join(Settings.tempDirectory, "data.xml"))

		self.assertEqual(experiment_imported.getMaxTime(), 3600.0)
		self.assertEqual(experiment_imported.getTimes(), [0.0, 900.0, 1800.0, 300.0, 3600.0, 180.0, 60.0])
		self.assertEqual(experiment_imported.getTreatedVariables(), ['FGF2'])

		experiment_imported.writeNuMLToFile(join(Settings.tempDirectory, "data_copy.xml"))

	def testAttribute(self):

		experiment = Experiment("Ras-GTP, ERK1-2 quantifications")
		experiment.notes = "Elisa quantifications for RAS-GTP, Western blot quanfitication for ERK1-2"
		condition_0 = experiment.createCondition("Starved")

		condition_0.addInitialCondition(0, "fgf2", 0, name_attribute="id")
		condition_0.notes = "starved cells for 48h"
		condition_0.addObservation(0, "total_ras_gtp", 40, name_attribute="id")
		condition_0.addObservation(60, "total_ras_gtp", 40, name_attribute="id")
		condition_0.addObservation(180, "total_ras_gtp", 40, name_attribute="id")
		condition_0.addObservation(300, "total_ras_gtp", 40, name_attribute="id")
		condition_0.addObservation(900, "total_ras_gtp", 40, name_attribute="id")
		condition_0.addObservation(1800, "total_ras_gtp", 40, name_attribute="id")
		condition_0.addObservation(3600, "total_ras_gtp", 40, name_attribute="id")

		condition_1 = experiment.createCondition("fgf2")
		condition_1.notes = "FGF2 treatment"
		condition_1.addInitialCondition(0, "fgf2", 333, name_attribute="id")

		condition_1.addObservation(0, "total_ras_gtp", 40, name_attribute="id")
		condition_1.addObservation(60, "total_ras_gtp", 92, name_attribute="id")
		condition_1.addObservation(180, "total_ras_gtp", 77, name_attribute="id")
		condition_1.addObservation(300, "total_ras_gtp", 333, name_attribute="id")
		condition_1.addObservation(900, "total_ras_gtp", 222, name_attribute="id")
		condition_1.addObservation(1800, "total_ras_gtp", 68, name_attribute="id")
		condition_1.addObservation(3600, "total_ras_gtp", 60, name_attribute="id")

		experiment.writeNuMLToFile(join(Settings.tempDirectory, "data_with_ids.xml"))

		experiment_imported = Experiment()
		experiment_imported.readNuMLFromFile(join(Settings.tempDirectory, "data_with_ids.xml"))

		self.assertEqual(experiment_imported.getMaxTime(), 3600.0)
		self.assertEqual(experiment_imported.getTimes(), [0.0, 900.0, 1800.0, 300.0, 3600.0, 180.0, 60.0])
		self.assertEqual(experiment_imported.getTreatedVariables(), ['fgf2'])

		experiment_imported.writeNuMLToFile(join(Settings.tempDirectory, "data_with_ids_copy.xml"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.sedml.HasId import HasId
from libsignetsim.sedml.XPath import XPath
from libsignetsim.settings.Settings import Settings


class Change(SedBase, HasId):

	def __init__(self, document):

		SedBase.__init__(self, document)
		HasId.__init__(self, document)

		self.__document = document
		self.__target = XPath(self.__document)

	def readSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, change, level, version)
		HasId.readSedml(self, change, level, version)

		if change.isSetTarget():
			self.__target.readSedml(change.getTarget(), level, version)

	def writeSedml(self, change, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, change, level, version)
		HasId.writeSedml(self, change, level, version)

		if self.__target.getXPath() is not None:
			change.setTarget(self.__target.writeSedml(level, version))

	def getTarget(self):
		return self.__target

	def setTarget(self, target):
		self.__target.setModelObject(target)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.settings.Settings import Settings

from libsignetsim.cwriter.CWriterModels import CWriterModels
from libsignetsim.cwriter.CWriterData import CWriterData
from libsignetsim.cwriter.CWriterOptimization import CWriterOptimization

class CWriterModelVsDataOptimization(CWriterOptimization, CWriterModels, CWriterData):

	def __init__(self, workingModel, listOfExperiments=None, mapping=None, parameters_to_fit=None,
				p_lambda=Settings.defaultPlsaLambda,
				p_criterion=Settings.defaultPlsaCriterion,
				p_precision=Settings.defaultPlsaPrecision,
				p_initial_temperature=Settings.defaultPlsaInitialTemperature,
				p_gain=Settings.defaultPlsaGainForJumpSizeControl,
				p_interval=Settings.defaultPlsaInterval,
				p_mix=Settings.defaultPlsaMixInterval,
				p_initial_moves=Settings.defaultPlsaInitialMoves,
				p_tau=Settings.defaultPlsaTau,
				p_freeze_count=Settings.defaultPlsaFreezeCount,
				s_neg_penalty=Settings.defaultScoreNegativePenalty,
	):


		self.listOfExperiments = listOfExperiments

		self.timeMin = None
		self.timeMax = None
		self.listOfSamples = None
		self.scoreNegativePenalty = s_neg_penalty
		self.findSimulationSettings()

		CWriterModels.__init__(self, [workingModel], [Settings.simulationTimeMin], [self.listOfSamples], [Settings.defaultAbsTol], [Settings.defaultRelTol])
		CWriterData.__init__(self, listOfExperiments, mapping, workingModel, subdir="src", hasObservations=True)
		CWriterOptimization.__init__(
			self, workingModel, parameters_to_fit,
			p_lambda=p_lambda, p_criterion=p_criterion, p_precision=p_precision,
			p_initial_temperature=p_initial_temperature,
			p_gain=p_gain, p_interval=p_interval, p_tau=p_tau,
			p_mix=p_mix, p_initial_moves=p_initial_moves,
			p_freeze_count=p_freeze_count
		)

	def findSimulationSettings(self):

		# We need to start it at zero, even if the first observation is laters
		times = [0.0]
		for experiment in self.listOfExperiments:
			times += experiment.getTimes()

		self.listOfSamples = sorted(list(set(times)))
		self.timeMin = min(self.listOfSamples)
		self.timeMax = max(self.listOfSamples)

	def writeOptimizationFiles(self, nb_procs):

		CWriterOptimization.writeOptimizationFiles(self, nb_procs)
		self.writeModelFiles()
		self.writeDataFiles()
		self.writeOptimFiles(nb_procs)

	def writeOptimFiles(self, nb_procs):

		c_filename = self.getTempDirectory() + Settings.C_generatedDirectory_v2 + "optim.c"
		h_filename = self.getTempDirectory() + Settings.C_generatedDirectory_v2 + "optim.h"

		f_c = open(c_filename, 'w')
		f_h = open(h_filename, 'w')

		self.writeOptimizationFilesHeaders(f_c, f_h)
		self.writeOptimizationGlobals(f_c, f_h)
		self.writeOptimizationGlobalMethods(f_c, f_h)
		self.writeOptimizationParameters(f_c, f_h)
		self.writeOptimizationSettings(f_c, f_h)
		self.writeOptimizationScoreSettings(f_c, f_h)

		f_c.close()
		f_h.close()



	def writeOptimizationScoreSettings(self, f_c, f_h):

		f_h.write("ScoreSettings * init_score_settings();\n")
		f_c.write("ScoreSettings * init_score_settings()\n{\n")

		f_c.write("  ScoreSettings * settings = malloc(sizeof(ScoreSettings));\n")
		f_c.write("  settings->negative_penalty = %.2g;\n" % self.scoreNegativePenalty)
		f_c.write("  return settings;\n")
		f_c.write("}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.sedml.HasId import HasId
from libsignetsim.settings.Settings import Settings


class DataSet(SedBase, HasId):

	def __init__(self, document):

		SedBase.__init__(self, document)
		HasId.__init__(self, document)

		self.__document = document
		self.__dataReference = None
		self.__label = None

		self.__data = None

	def readSedml(self, data_set, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, data_set, level, version)
		HasId.readSedml(self, data_set, level, version)

		if data_set.isSetDataReference():
			self.__dataReference = data_set.getDataReference()

		if data_set.isSetLabel():
			self.__label = data_set.getLabel()

	def writeSedml(self, data_set, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, data_set, level, version)
		HasId.writeSedml(self, data_set, level, version)

		if self.__dataReference is not None:
			data_set.setDataReference(self.__dataReference)

		if self.__label is not None:
			data_set.setLabel(self.__label)

	def getDataReference(self):
		return self.__dataReference

	def getLabel(self):
		return self.__label

	def setDataReference(self, data_reference):
		self.__dataReference = data_reference

	def setData(self, data):
		self.__dataReference = data.getId()

	def setLabel(self, label):
		self.__label = label

	def getData(self):

		data_generator = self.__document.listOfDataGenerators.getDataGenerator(self.__dataReference)
		return {self.__label: data_generator.getData()}

	def getDataToGenerate(self):

		return self.__document.listOfDataGenerators.getDataGenerator(self.__dataReference)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.HasId import HasId
from libsignetsim.settings.Settings import Settings
from sympy import Symbol

class SbmlVariable(HasId):

	SPECIES             = 0
	COMPARTMENT         = 1
	PARAMETER           = 2
	STOICHIOMERY        = 3
	REACTION            = 4
	EVENT				= 5
	CONSERVED_MOIETY    = 6
	def __init__(self, model, sbml_type, is_from_reaction=None):

		self.__model = model
		HasId.__init__(self, model)
		self.sbmlType = sbml_type
		self.reaction = is_from_reaction

	def new(self, name, sbml_type=PARAMETER):

		self.sbmlType = sbml_type
		t_sbmlId = self.__model.listOfVariables.addVariable(self, name)
		HasId.new(self, name, t_sbmlId)

	def copy(self, obj, sids_subs={}):

		HasId.copy(self, obj, sids_subs=sids_subs)

		if self.isParameter() and self.localParameter:
			self.__model.listOfVariables.addVariable(self,
			"_local_%d_%s" % (self.reaction.objId, self.getSbmlId()))

		else:
			self.__model.listOfVariables.addVariable(self, self.getSbmlId())
		self.sbmlType = obj.sbmlType

	def readSbml(self, sbml_variable,
						sbml_level=Settings.defaultSbmlLevel,
						sbml_version=Settings.defaultSbmlVersion):

		HasId.readSbml(self, sbml_variable, sbml_level, sbml_version)

		if self.isParameter() and self.localParameter:
			self.__model.listOfVariables.addVariable(self,
					"_local_%d_%s" % (self.reaction.objId, self.getSbmlId()))
		else:
			self.__model.listOfVariables.addVariable(self, self.getSbmlId())


	def writeSbml(self, sbml_variable,
						sbml_level=Settings.defaultSbmlLevel,
						sbml_version=Settings.defaultSbmlVersion):

		HasId.writeSbml(self, sbml_variable, sbml_level, sbml_version)


	def isSpecies(self):
		return self.sbmlType == self.SPECIES

	def isParameter(self):
		return self.sbmlType == self.PARAMETER

	def isCompartment(self):
		return self.sbmlType == self.COMPARTMENT

	def isStoichiometry(self):
		from libsignetsim.model.sbml.SpeciesReference import SpeciesReference

		return isinstance(self, SpeciesReference)

	def isReaction(self):
		return self.sbmlType == self.REACTION

	def isEvent(self):
		return self.sbmlType == self.EVENT

	def isConservedMoiety(self):
		return self.sbmlType == self.CONSERVED_MOIETY

	def isGlobal(self):
		return self.reaction is None


	def setSbmlId(self, sbml_id, prefix="", model_wide=True):

		# print "Inside setSbmlId"
		if self.isParameter() and self.isLocalParameter():
			# print "is local parameter"
			self.reaction.renameSbmlId(self.getSbmlId(), sbml_id)
			HasId.setSbmlId(self, sbml_id)
		else:
			HasId.setSbmlId(self, sbml_id, prefix, model_wide)

	def renameSbmlId(self, sbml_id):
		HasId.setSbmlId(self, sbml_id, model_wide=False, list_of_var=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from future import standard_library
standard_library.install_aliases()

from libsignetsim.sedml.URI import URI
from libsignetsim.sedml.SedmlException import SedmlFileNotFound
from libsignetsim.settings.Settings import Settings
from six.moves.urllib.request import urlretrieve
from os.path import join, exists, isabs, relpath, commonprefix, basename
from os import getcwd

class Source(object):

	LOCAL = 0
	DISTANT = 1
	URN = 2
	REF = 3

	def __init__(self, document):
		self.__document = document
		self.__source = None
		self.__sourceType = None
		self.__sourceAvailable = False
		self.__filename = None
		self.__url = None
		self.__uri = URI(self.__document)

	def readSedml(self, source, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		self.__source = source
		self.parseSource(source)

	def writeSedml(self, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		if self.__source is not None:
			return self.__source

	def parseSource(self, source):

		if source.startswith("http:"):
			self.__sourceType = self.DISTANT
			self.__url = source
			self.__filename = join(Settings.tempDirectory, source.split('/')[-1])

		elif source.startswith("urn:"):

			self.__sourceType = self.URN
			self.__uri.setURI(source)
			if self.__uri.getURL():
				self.__url = self.__uri.getURL()
				self.__filename = join(Settings.tempDirectory, self.__uri.getFilename())

		elif self.__document.listOfModels.getModelByReference(source) is not None:
			self.__sourceType = self.REF
			self.__ref = source
			# self.__filename = self.__document.listOfModels.getModelByReference(source).getSource().getFilename()
			# self.__sourceAvailable = True
		else:
			self.__sourceType = self.LOCAL
			self.__sourceAvailable = True
			if isabs(source) or exists(join(getcwd(), source)):
				self.__filename = source

			elif self.__document.path is not None:
				self.__filename = join(self.__document.path, source)
			else:
				raise SedmlFileNotFound("File not found : %s" % source)

	def buildSource(self):

		if self.__sourceType == self.LOCAL:
			return self.__source

		elif self.__sourceType == self.DISTANT:
			return self.__url

		elif self.__sourceType == self.URN:
			return self.__uri.writeSedml()

		elif self.__sourceType == self.REF:
			return self.__ref


	def downloadSource(self):

		urlretrieve(self.__url, self.__filename)
		self.__sourceAvailable = True

	def getSource(self):
		return self.__source

	def getFilename(self):
		# print "Getting filename for %s" % self.__source

		if self.__sourceType == self.REF:
			return self.__document.listOfModels.getModelByReference(self.__source).getSource().getFilename()
		elif not self.__sourceAvailable:
			self.downloadSource()
		return self.__filename

	def setSource(self, source):
		self.__source = source
		self.parseSource(source)

	def isLocal(self):
		return self.__sourceType == self.LOCAL

	def makeRelativePath(self, path):

		if self.__sourceType == self.LOCAL and (isabs(self.__source) or commonprefix([self.__source, path]) != ""):
			self.__source = relpath(self.__source, path)

	def getSourceType(self):
		return self.__sourceType

	def removePaths(self):

		if self.__sourceType == self.LOCAL:
			self.__filename = basename(self.__filename)
			self.__source = basename(self.__source)

	def makeLocalSource(self):

		if self.__sourceType in [self.URN, self.DISTANT]:

			t_filename = self.getFilename()
			self.__sourceType = self.LOCAL
			self.__filename = t_filename.split('/')[-1]
			self.__sourceAvailable = True
			return t_filename



#!/usr/bin/env python

from setuptools import find_packages
from distutils.core import setup, Extension
from os.path import dirname, join

setup(name='libsignetsim',
	version=open(join(dirname(__file__), 'VERSION')).read(),
	description='A library designed for building, adjusting and analyzing quantitative biological models. ',
	author='Vincent Noel',
	author_email='contact@vincent-noel.fr',
	url='',
	packages=find_packages(),
	include_package_data=True,
	install_requires=[
		'matplotlib',
		'python-libsbml',
		'python-libnuml',
		'python-libsedml',
		'sympy<1.2',
		'numpy',
		'scipy',
		'pydstool',
		'jinja2',
		'bioservices',
		'lxml',
		'coveralls',
		'future'
	],
	ext_modules=[
		Extension(
			'libsignetsim.lib.integrate.integrate',
			sources=[
				'libsignetsim/lib/integrate/src/shared.c',
				'libsignetsim/lib/integrate/src/events.c',
				'libsignetsim/lib/integrate/src/ode.c',
				'libsignetsim/lib/integrate/src/dae.c',
				'libsignetsim/lib/integrate/src/integrate.c',
				'libsignetsim/lib/integrate/src/realtype_math.c',
			],
			include_dirs=[
				"/usr/include/cvode/",
				"/usr/include/ida/",
				"/usr/include/nvector"
				"/usr/include/sundials",
			],
			libraries=['sundials_cvode', 'sundials_nvecserial', 'sundials_ida', 'm', 'lapack', 'atlas', 'blas'],
			library_dirs=['/usr/lib64/atlas-basic/'],
			define_macros=(
				[("SUNDIALS3", None)] if "SUNDIALS_VERSION_MAJOR 3" in open("/usr/include/sundials/sundials_config.h").read() else None
			),
		),
		Extension(
			'libsignetsim.lib.plsa.libplsa-serial',
			sources=[
				'libsignetsim/lib/plsa/src/config.c',
				'libsignetsim/lib/plsa/src/error.c',
				'libsignetsim/lib/plsa/src/distributions.c',
				'libsignetsim/lib/plsa/src/random.c',
				'libsignetsim/lib/plsa/src/plsa.c',
				'libsignetsim/lib/plsa/src/lsa.c',
				'libsignetsim/lib/plsa/src/moves.c',
				'libsignetsim/lib/plsa/src/state.c',
				'libsignetsim/lib/plsa/src/score.c',
			]
		),
		Extension(
			'libsignetsim.lib.plsa.libplsa-parallel',
			sources=[
				'libsignetsim/lib/plsa/src/config.c',
				'libsignetsim/lib/plsa/src/error.c',
				'libsignetsim/lib/plsa/src/distributions.c',
				'libsignetsim/lib/plsa/src/random.c',
				'libsignetsim/lib/plsa/src/plsa.c',
				'libsignetsim/lib/plsa/src/lsa.c',
				'libsignetsim/lib/plsa/src/moves.c',
				'libsignetsim/lib/plsa/src/state.c',
				'libsignetsim/lib/plsa/src/score.c',
				'libsignetsim/lib/plsa/src/mixing.c',
				'libsignetsim/lib/plsa/src/tuning.c',
			],
			include_dirs=[
				"/usr/lib/openmpi/include/openmpi/opal/mca/event/libevent2021/libevent",
				"/usr/lib/openmpi/include/openmpi/opal/mca/event/libevent2021/libevent/include",
				"/usr/lib/openmpi/include",
				"/usr/lib/openmpi/include/openmpi",
				"/usr/lib/x86_64-linux-gnu/openmpi/include/openmpi/opal/mca/event/libevent2021/libevent",
				"/usr/lib/x86_64-linux-gnu/openmpi/include/openmpi/opal/mca/event/libevent2021/libevent/include",
				"/usr/lib/x86_64-linux-gnu/openmpi/include",
				"/usr/lib/x86_64-linux-gnu/openmpi/include/openmpi",
				"/usr/lib64/mpi/gcc/openmpi/include/", # Opensuse
				"/usr/include/openmpi-x86_64/", # Fedora
			],
			define_macros=[("MPI", None)],
			extra_compile_args=["-pthread"]
		)
	],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""

from libsignetsim import SedmlDocument

from unittest import TestCase
from os.path import join, dirname, isdir
from os import mkdir


class TestBiomodelsURI(TestCase):
	""" Tests high level functions """

	def testBiomodelsURI(self):

		sedml_doc = SedmlDocument()

		model = sedml_doc.listOfModels.createModel()
		model.setLanguageSbml()
		model.setSource("urn:miriam:biomodels.db:BIOMD0000000048")

		sbml_model = model.getSbmlModel()
		self.assertEqual(sbml_model.getName(), "Kholodenko1999 - EGFR signaling")








#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the module SigNetSim

"""
from __future__ import absolute_import, unicode_literals, print_function

import matplotlib
matplotlib.use("agg", warn=False)

from .combine import (
	CombineArchive, CombineException, FileNotFoundException, NotAZipFileException, NoManifestFoundException,
	NoMasterSedmlFoundException, NoMasterSbmlFoundException, NoMasterNumlFoundException, NoSedmlFoundException
)
from .settings import Settings
from .continuation import EquilibriumPointCurve
from .data import Experiment
from .numl import NuMLDocument, NuMLException, NuMLFileNotFound
from .sedml import (
	SedmlDocument, SedmlException, SedmlMathException, SedmlFileNotFound, SedmlMixedSubtasks,
	SedmlModelLanguageNotSupported, SedmlModelNotFound, SedmlModelObjectNotFound, SedmlMultipleModels,
	SedmlNotImplemented, SedmlOneStepTaskException, SedmlUnknownURI, SedmlUnknownXPATH
)
from .simulation import (
	TimeseriesSimulation, SteadyStatesSimulation, SimulationCompilationException, SimulationExecutionException,
	SimulationNoDataException, SimulationException
)

from .optimization import ModelVsTimeseriesOptimization, OptimizationException

from .model import (
	Model, SbmlDocument, MathFormula, KineticLaw,
	ModelException, SbmlException, FileException,
	MissingModelException, MissingSubmodelException,
	PackageNotImplementedModelException, TagNotImplementedModelException,
	UnknownSIdRefException, InvalidXPath,
	CannotCreateException, CannotDeleteException,

)

from .figure import SigNetSimFigure

from .LibSigNetSimException import LibSigNetSimException


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""



from libsignetsim.model.math.sympy_shortcuts import (
	SympySymbol, SympyInteger, SympyFloat, SympyPi, SympyMul, SympyPow, SympyRateOf, SympyUndefinedFunction,
	SympyTrue, SympyFalse, SympyExprCondPair, SympyITE, SympyLambda, SympyTuple, SympyAvogadro)
from re import match
from sympy import srepr


def unevaluatedSubs(expr, substitutions, *args, **kwargs):
	"""Crawl the expression tree, and apply func to every node."""
	if len(substitutions) == 0:
		return expr

	val = simpleSubs(expr, substitutions, *args, **kwargs)
	if val is not None:
		return val
	new_args = (unevaluatedSubs(arg, substitutions, *args, **kwargs) for arg in expr.args)

	if expr.func in [SympyExprCondPair, SympyITE]:
		return expr.func(*new_args)
	else:
		return expr.func(*new_args, evaluate=False)

def simpleSubs(expr, substitutions, *args, **kwarg):
	"""Perform expression substitution, ignoring derivatives."""
	if expr in substitutions:
		return substitutions[expr]
	elif not expr.args:
		return expr


class MathDevelopper(object):
	""" Class for handling math formulaes """

	def __init__(self, model):
		""" Constructor """

		self.__model = model

	def translateVariableForDeveloppedInternal(self, variable):

		if str(variable).startswith("_speciesForcedConcentration_"):
			res_match = match(r"_speciesForcedConcentration_(.*)_", str(variable))

			t_sbml_id = str(res_match.groups()[0])
			t_species = self.__model.listOfVariables.getBySymbol(SympySymbol(t_sbml_id))
			t_compartment = t_species.getCompartment()

			return SympyMul(t_species.symbol.getInternalMathFormula(),
							SympyPow(t_compartment.symbol.getInternalMathFormula(),
										SympyInteger(-1)))

		elif str(variable).startswith("_functionDefinition_"):
			res_match = match(r"_functionDefinition_(\d+)_", str(variable))
			t_id = int(res_match.groups()[0])
			t_definition = self.__model.listOfFunctionDefinitions[t_id].getDefinition().getInternalMathFormula()

			return self.translateForDeveloppedInternal(t_definition)

		else:

			return variable


	def translateForDeveloppedInternal(self, tree):

		if tree is None:
			return None

		if tree in [True, False]:
			return tree

		if tree in [SympyTrue, SympyFalse, SympyPi] or tree.func in [SympyInteger, SympyFloat]:
			return tree


		if tree.func == SympyRateOf:

			if str(tree.args[0]).startswith("_speciesForcedConcentration_"):
				res_match = match(r"_speciesForcedConcentration_(.*)_", str(tree.args[0]))

				t_sbml_id = str(res_match.groups()[0])
				t_variable = self.__model.listOfVariables.getBySymbol(SympySymbol(t_sbml_id))

				if t_variable.isSpecies() and not t_variable.hasOnlySubstanceUnits:# and not t_variable.isRateRuled():
					t_ode = t_variable.getODE().getDeveloppedInternalMathFormula()
					if t_ode is not None:
						if not t_variable.isRateRuled():
							t_ode /= t_variable.getCompartment().symbol.getDeveloppedInternalMathFormula()
						elif t_variable.getCompartment().isRateRuled():
							t_amount_species = t_variable.symbol.getInternalMathFormula()
							t_comp_rate = t_variable.getCompartment().isRuledBy().getDefinition().getInternalMathFormula()
							t_comp = t_variable.getCompartment().symbol.getInternalMathFormula()
							t_ode = t_ode - t_amount_species * t_comp_rate / t_comp

				else:
					t_ode = t_variable.getODE().getDeveloppedInternalMathFormula()

			else:
				t_variable = self.__model.listOfVariables.getBySymbol(tree.args[0])
				t_ode = t_variable.getODE().getDeveloppedInternalMathFormula()

			if t_ode is None:
				return SympyInteger(0)

			return t_ode

		if isinstance(tree.func, SympyUndefinedFunction) and "_functionDefinition_" in str(tree.func):
			res_match = match(r"_functionDefinition_(\d+)_", str(tree.func))
			t_id = int(res_match.groups()[0])
			t_definition = self.translateForDeveloppedInternal(self.__model.listOfFunctionDefinitions[t_id].getDefinition().getInternalMathFormula().args[1])
			t_arguments = list(self.__model.listOfFunctionDefinitions[t_id].getDefinition().getInternalMathFormula().args[0])

			subs = {}
			for child in range(0, len(tree.args)):
				subs.update({t_arguments[child]: self.translateForDeveloppedInternal(tree.args[child])})

			return unevaluatedSubs(t_definition, subs)

		elif isinstance(tree, SympyUndefinedFunction):
			res_match = match(r"_functionDefinition_(\d+)_", str(tree))
			t_id = int(res_match.groups()[0])
			t_definition = self.__model.listOfFunctionDefinitions[t_id].getDefinition().getInternalMathFormula()

			return self.translateForDeveloppedInternal(t_definition)

		elif tree.func == SympySymbol:
			return self.translateVariableForDeveloppedInternal(tree)

		elif tree.func == SympyLambda and len(tree.args) > 0 and tree.args[0] == SympyTuple():
			return self.translateForDeveloppedInternal(tree.args[1])

		else:
			if len(tree.args) > 0:
				t_children = []
				for child in range(0, len(tree.args)):
					t_children.append(self.translateForDeveloppedInternal(tree.args[child]))

				# Seems like not evaluating is not an option for these ones...
				if tree.func in [SympyExprCondPair, SympyITE]:
					return tree.func(*tuple(t_children))

				return tree.func(*tuple(t_children), evaluate=False)

			else:
				return tree




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.model.math.DAE import DAE
from libsignetsim.model.math.CFE import CFE
from libsignetsim.model.math.ODE import ODE
from libsignetsim.model.math.MathSubmodel import MathSubmodel
from libsignetsim.model.math.MathVariable import MathVariable
from libsignetsim.model.math.MathFormula import MathFormula
from sympy import solve, ones
from random import choice


class MathAsymmetricModel(MathSubmodel):
	""" Sbml model class """

	def __init__(self, parent_model=None):
		""" Constructor of model class """

		MathSubmodel.__init__(self, parent_model=parent_model)
		self.DEBUG = False

	def copyEquations(self):

		for cfe in self.parentModel.listOfCFEs:
			t_var = self.listOfVariables.getBySymbol(cfe.getVariable().symbol.getSymbol())
			t_cfe_formula = MathFormula(self)
			t_cfe_formula.setInternalMathFormula(cfe.getDefinition().getDeveloppedInternalMathFormula())
			t_cfe = CFE(self)
			t_cfe.new(t_var, t_cfe_formula)
			self.listOfCFEs.append(t_cfe)

		for dae in self.parentModel.listOfDAEs:
			t_dae_formula = MathFormula(self)
			t_dae_formula.setInternalMathFormula(dae.getDefinition().getDeveloppedInternalMathFormula())
			t_dae = DAE(self)
			t_dae.new(t_dae_formula)
			self.listOfDAEs.append(t_dae)

		self.copyEvents()
		self.copyInitialAssignments()

	def build(self, treated_variables=[]):

		if not self.parentModel.listOfConservationLaws.isUpToDate():
			self.parentModel.listOfConservationLaws.build()

		independent_species = []
		independent_species_formula = {}

		if len(self.parentModel.listOfConservationLaws) > 0:

			allowed_variables, forbidden_variables = self.__findAllowedVariables(treated_variables)
			allowed_laws, forbidden_laws = self.__findAllowedLaws()
			dependent_vars, independent_species = self.__findReducibleVariables(allowed_laws, allowed_variables)

			if self.DEBUG:
				print("> Dependent : %s" % dependent_vars)
				print("> Independent : %s" % independent_species)

			independent_species, independent_species_formula = self.__solveDependentVariables(forbidden_laws, dependent_vars)

			if self.DEBUG:
				print("> %d variables reduced from the model" % len(dependent_vars))

		self.__buildModel(independent_species, independent_species_formula)

	def __findAllowedVariables(self, treated_variables):

		all_variables = list(range(len(self.parentModel.variablesOdes)))

		forbidden_variables = []
		for i_species, species in enumerate(self.parentModel.variablesOdes):
			if (
				species.getSymbolStr() in treated_variables
				or species.hasEventAssignment()
				or species.hasInitialAssignment()
			):
				forbidden_variables.append(i_species)

		allowed_variables = list(set(all_variables).difference(set(forbidden_variables)))
		return allowed_variables, forbidden_variables

	def __findAllowedLaws(self):

		forbidden_laws = []
		for i, law in enumerate(self.parentModel.listOfConservationLaws):
			for var in law.vars:
				variable = self.parentModel.listOfVariables.getBySymbol(var)
				if variable.hasEventAssignment() or variable.hasInitialAssignment():
					forbidden_laws.append(i)

		if self.DEBUG:
			print("%d laws forbidden (%s)" % (len(forbidden_laws), str(forbidden_laws)))

		allowed_laws = list(set(range(len(self.parentModel.listOfConservationLaws))).difference(set(forbidden_laws)))
		return allowed_laws, forbidden_laws

	def __findReducibleVariables(self, allowed_laws, allowed_variables):

		cons_matrix = self.parentModel.conservationMatrix.getConservationMatrix()
		cons_matrix_allowed = cons_matrix[allowed_laws, allowed_variables]

		# Here we are trying to find groups of symmetrical variables.
		# The idea is that we first look at the number of time a species is in a conservation law.
		# Then, we group withing each law by the number of time it is in a conservation law modelwide
		# Each group is added to a list of global groups
		times_in_cons = ones(1, cons_matrix_allowed.shape[0]) * cons_matrix_allowed

		groups = []
		for i_cons in range(cons_matrix.shape[0]):

			i_matrix = cons_matrix[i_cons, allowed_variables]
			i_time_in_cons = i_matrix.multiply_elementwise(times_in_cons)

			t_dict = {int(val): [] for val in set(list(i_time_in_cons)) if val > 0}
			for i, val in enumerate(i_time_in_cons):
				if val > 0:
					t_dict[int(val)].append(i)

			for nb, species in list(t_dict.items()):
				if species not in groups:
					groups.append(species)

		# Once we have these groups, we take one if each of them
		dependent_vars = []
		for group in groups:
			dependent_vars.append(allowed_variables[choice(group)])

		all_variables = list(range(len(self.parentModel.variablesOdes)))

		independent_vars = list(set(all_variables).difference(set(dependent_vars)))

		return dependent_vars, independent_vars

	def __solveDependentVariables(self, forbidden_laws, dependent_vars):

		system = []
		vars = []

		i_var = 0

		for i_law, cons_law in enumerate(self.parentModel.listOfConservationLaws):
			if i_law not in forbidden_laws and i_var < len(dependent_vars):
				system.append(cons_law.getFormula())

				dependent_species = self.parentModel.variablesOdes[dependent_vars[i_var]]
				vars.append(dependent_species.symbol.getSymbol())
				i_var += 1

		if self.DEBUG:
			print(system)
			print(vars)

		result_system = solve(system, vars)

		if self.DEBUG:
			print(result_system)

		independent_species = []
		independent_species_formula = []

		if len(result_system) > 0:
			if isinstance(result_system, dict):
				for var, value in list(result_system.items()):
					independent_species.append(var)
					independent_species_formula.append(value)
			else:
				print(result_system)

		return independent_species, independent_species_formula

	def __buildModel(self, independent_species, independent_species_formula):

		self.clear()
		self.copyVariables()
		self.copyEquations()

		for var in self.parentModel.variablesOdes:
			new_var = self.listOfVariables.getBySymbol(var.symbol.getSymbol())
			if var.symbol.getSymbol() in independent_species:

				t_formula = MathFormula(self)

				t_formula.setInternalMathFormula(
					independent_species_formula[independent_species.index(var.symbol.getSymbol())]
				)

				t_cfe = CFE(self)
				t_cfe.new(new_var, t_formula)
				self.listOfCFEs.append(t_cfe)
				self.listOfVariables.changeVariableType(new_var, MathVariable.VAR_ASS)

			else:

				t_formula = MathFormula(self)
				t_formula.setInternalMathFormula(
					self.parentModel.listOfODEs.getByVariable(
						var).getDefinition().getDeveloppedInternalMathFormula()
				)

				t_ode = ODE(self)
				t_ode.new(new_var, t_formula)
				self.listOfODEs.append(t_ode)

		self.listOfCFEs.developCFEs()
		self.setUpToDate(True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.sbml.SbmlModel import SbmlModel
from libsignetsim.model.math.MathModel import MathModel
from libsignetsim.model.ListOfVariables import ListOfVariables
from libsignetsim.settings.Settings import Settings
from time import time

class Model(SbmlModel, MathModel):
	""" Sbml model class """

	def __init__(self, obj_id=0, parent_doc=None, parent_obj=None):

		""" Constructor of model class """

		self.objId = obj_id

		SbmlModel.__init__(self, parent_doc, parent_obj, obj_id)
		MathModel.__init__(self, obj_id)
		self.listOfVariables = ListOfVariables(self)

	def build(self, vars_to_keep=[], reduce=Settings.reduceByDefault, tmin=0):

		t0 = time()
		self.listOfVariables.classifyVariables()
		MathModel.buildModel(self, vars_to_keep=vars_to_keep, reduce=reduce, tmin=tmin)

		if Settings.verboseTiming >= 1:
			print(">> Model built in %.2gs" % (time()-t0))

	def cleanBeforePickle(self):
		# self.listOfVariables.cleanFinal()
		pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.MathVariable import MathVariable
from libsignetsim.model.math.DAE import DAE
from libsignetsim.model.math.CFE import CFE
from libsignetsim.model.math.sympy_shortcuts import SympyEqual, SympyInteger, SympySymbol, SympyFloat
from libsignetsim.model.math.MathException import MathException
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs

from sympy import solve, srepr
from time import time

class ListOfDAEs(list):
	""" Sbml model class """

	def __init__(self, model):
		""" Constructor of model class """

		self.__model = model
		list.__init__(self)

	def build(self):

		for rule in self.__model.listOfRules:
			if rule.isAlgebraic() and rule.isValid():
				# self.__model.hasDAEs = True
				t_dae = DAE(self.__model)
				t_dae.new(rule.getDefinition(rawFormula=True))
				list.append(self, t_dae)

	def solveInitialConditions(self, tmin=0):

		DEBUG = False
		system = []

		subs = {SympySymbol('time'): SympyFloat(tmin)}
		for var, val in list(self.__model.listOfInitialConditions.items()):
			if not self.__model.listOfVariables.getBySymbol(var).isAlgebraic():
				subs.update({var: val.getInternalMathFormula()})

		for dae in self:
			system.append(
				SympyEqual(
					unevaluatedSubs(dae.getDefinition().getDeveloppedInternalMathFormula(), subs),
					SympyInteger(0)
				)
			)

		system_vars = []
		for var in self.__model.listOfVariables:
			if var.isAlgebraic():
				system_vars.append(var.symbol.getInternalMathFormula())

		if DEBUG:
			print(system)
			print(system_vars)

		all_true = True
		for equ in system:
			if equ != True:
				all_true = False

		if len(system_vars) > 0 and not all_true:

			init_cond = {}
			res = solve(system, system_vars, manual=True)

			if DEBUG:
				print(res)

			if res is not True and len(res) > 0:
				if isinstance(res, dict):
					for var, value in list(res.items()):
						init_cond.update({var: value})

				elif isinstance(res[0], dict):
					for var, value in list(res[0].items()):
						init_cond.update({var: value})

				elif isinstance(res[0], tuple):
					for i_var, value in enumerate(res[0]):
						init_cond.update({system_vars[i_var]: value})

			if DEBUG:
				print(init_cond)

			t0 = time()

			if tmin == 0:
				init_cond.update({SympySymbol("_time_"): SympyInteger(tmin)})
			else:
				init_cond.update({SympySymbol("_time_"): SympyFloat(tmin)})

			for init_ass in self.__model.listOfInitialAssignments:
				if init_ass.isValid():
					t_var = init_ass.getVariable().symbol.getSymbol()
					t_value = init_ass.getDefinition().getDeveloppedInternalMathFormula()
					init_cond.update({t_var:t_value})

			if DEBUG:
				print(init_cond)

			for rule in self.__model.listOfRules:
				if rule.isAssignment() and rule.isValid():
					t_var = rule.getVariable().symbol.getSymbol()

					if t_var not in list(init_cond.keys()):
						t_value = rule.getDefinition().getDeveloppedInternalMathFormula()
						init_cond.update({t_var: t_value})

			if DEBUG:
				print(init_cond)

			for var in self.__model.listOfVariables:
				t_var = var.symbol.getSymbol()

				if t_var not in list(init_cond.keys()):
					t_value = var.value.getDeveloppedInternalMathFormula()
					if t_value is not None:
						init_cond.update({t_var: t_value})
					else:
						print("WTF %s" % var.getSbmlId())
			if DEBUG:
				print(init_cond)


			crossDependencies = True
			passes = 1
			while crossDependencies:
				if DEBUG:
					print("PASS : %d" % passes)
				crossDependencies = False
				for t_var in list(init_cond.keys()):
					t_def = init_cond[t_var]
					if len(t_def.atoms(SympySymbol).intersection(set(init_cond.keys()))) > 0:
						crossDependencies = True
						if DEBUG:
							print("\n> " + str(t_var) + " : " + str(t_def))

						for match in t_def.atoms(SympySymbol).intersection(set(init_cond.keys())):
							if match == t_var:
								raise MathException("Initial values : self dependency is bad")
							if DEBUG:
								print(">> " + str(match) + " : " + str(init_cond[match]))

							t_def = unevaluatedSubs(t_def, {match: init_cond[match]})
							init_cond.update({t_var: t_def})

						if DEBUG:
							if len(t_def.atoms(SympySymbol).intersection(set(init_cond.keys()))) == 0:
								print("> " + str(t_var) + " : " + str(t_def) + " [OK]")
							else:
								print("> " + str(t_var) + " : " + str(t_def) + " [ERR]")
				passes += 1
				if passes >= 100:
					raise MathException("Initial values : Probable circular dependencies")

				if DEBUG:
					print("")

			if DEBUG:
				print(list(init_cond.keys()))
				print(self.__model.listOfVariables.symbols())

			self.__model.listOfInitialConditions = {}
			for var, value in list(init_cond.items()):
				t_var = self.__model.listOfVariables.getBySymbol(var)
				if t_var is not None:
					t_value = MathFormula(self.__model)
					t_value.setInternalMathFormula(value)
					self.__model.listOfInitialConditions.update({t_var.symbol.getSymbol(): t_value})

			for var in self.__model.listOfVariables:
				if not var.symbol.getSymbol() in list(self.__model.listOfInitialConditions.keys()):
					print("Lacks an initial condition : %s" % var.getSbmlId())

	def solveDAEs(self):

		for i, dae in enumerate(self):
			if dae.isValid():
				var, res = dae.solve()

				if len(res) > 0:
					t_var = self.__model.listOfVariables.getBySymbol(var)
					t_formula = MathFormula(self.__model)

					if isinstance(res[0], dict):
						t_formula.setInternalMathFormula(list(res[0].values())[0])
					else:
						t_formula.setInternalMathFormula(res[0])

					cfe = CFE(self.__model)
					cfe.new(t_var, t_formula)
					self.__model.listOfCFEs.append(cfe)
					self.__model.listOfVariables.changeVariableType(t_var, MathVariable.VAR_ASS)
					list.__delitem__(self, i)

		self.__model.listOfCFEs.developCFEs()

	def __str__(self):

		res = ""
		for dae in self:
			res += str(dae) + "\n"

		return res


	def pprint(self):

		for dae in self:
			dae.pprint()
			print("\n")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.math.CFE import CFE
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.sympy_shortcuts import SympyPiecewise, SympyITE, SympyInf, SympyNan, SympyEqual, SympySymbol, SympyPow
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.MathException import MathException
from sympy import solve, sympify, srepr, simplify
from time import time

class ListOfCFEs(list):
	""" Sbml model class """


	def __init__ (self, model):
		""" Constructor of model class """

		self.__model = model
		list.__init__(self)

	def build(self):

		self[:] = []

		for rule in self.__model.listOfRules:
			if rule.isAssignment() and rule.isValid():
				t_cfe = CFE(self.__model, CFE.ASSIGNMENT)
				t_cfe.new(rule.getVariable(), rule.getDefinition(rawFormula=True))
				list.append(self, t_cfe)

		for reaction in self.__model.listOfReactions:
			if reaction.isValid():
				t_cfe = CFE(self.__model, CFE.REACTION)
				t_cfe.new(reaction, reaction.value)
				list.append(self, t_cfe)

		self.developCFEs()


	def developCFEs(self):

		DEBUG = False

		if DEBUG:
			print(self)

		t0 = time()

		if len(self) > 0:

			cfe_vars = []
			for t_cfe in self:
				cfe_vars.append(t_cfe.getVariable().symbol.getInternalMathFormula())

			if DEBUG:
				print(cfe_vars)
			crossDependencies = True
			passes = 1

			while crossDependencies:

				if DEBUG:
					print("PASS : %d" % passes)

				crossDependencies = False
				for t_cfe in self:
					t_def = t_cfe.getDefinition().getDeveloppedInternalMathFormula()
					if DEBUG:
						print(t_def)
					if len(t_def.atoms(SympySymbol).intersection(set(cfe_vars))) > 0:

						crossDependencies = True

						if DEBUG:
							print("\n> " + str(t_cfe))

						for match in t_def.atoms(SympySymbol).intersection(set(cfe_vars)):
							if match == t_cfe.getVariable().symbol:
								raise MathException("Developping CFEs : self dependency is bad")

							if DEBUG:
								print(">> " + str(self.getBySymbol(match)))

							t_cfe.setDefinitionMath(t_cfe.getDefinition().simpleSubsDevelopped(self.getBySymbol(match).getSubs()))

						if DEBUG:
							if len(t_cfe.getDefinition().getDeveloppedInternalMathFormula().atoms(SympySymbol).intersection(set(cfe_vars))) == 0:
								print("> " + str(t_cfe) + " [OK]")

							else:
								print("> " + str(t_cfe) + " [ERR]")

				passes += 1
				if passes >= 100:
					raise MathException("Developping CFEs : Probable circular dependencies")

				if DEBUG:
					print("")

		t1 = time()
		if Settings.verbose >= 1:
			print("> Finished developping closed forms (%.2gs)" % (t1-t0))

	def __str__(self):

		res = ""
		for t_cfe in self:
			res += str(t_cfe) + "\n"

		return res

	def pprint(self):

		for cfe in self:
			cfe.pprint()
			print("\n")

	def getByVariable(self, variable):

		for cfe in self:
			if variable == cfe.getVariable():
				return cfe

	def getBySymbol(self, symbol):
		for cfe in self:
			if symbol == cfe.getVariable().symbol.getInternalMathFormula():
				return cfe

	def getSubs(self):
		res = {}
		for cfe in self:
			res.update(cfe.getSubs())
		return res


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.sympy_shortcuts import  (
	SympySymbol, SympyInteger, SympyFloat, SympyRational, SympyAtom,
	SympyOne, SympyNegOne, SympyZero, SympyPi, SympyE, SympyExp1, SympyHalf,
	SympyInf, SympyNan, SympyAdd, SympyMul, SympyPow,
	SympyFunction, SympyUndefinedFunction, SympyLambda, SympyDerivative,
	SympyCeiling, SympyFloor, SympyAbs, SympyLog, SympyExp, SympyPiecewise,
	SympyFactorial, SympyRoot, SympyAcos, SympyAsin, SympyAtan, SympyAcosh,
	SympyAsinh, SympyAtanh, SympyCos, SympySin, SympyTan, SympyAcot,
	SympyAcoth, SympyCosh, SympySinh, SympyTanh, SympySec, SympyCsc,
	SympyCot, SympyCoth, SympyAcsc, SympyAsec,
	SympyEqual, SympyUnequal, SympyGreaterThan, SympyLessThan,
	SympyStrictGreaterThan, SympyStrictLessThan,
	SympyAnd, SympyOr, SympyXor, SympyNot, SympyTrue, SympyFalse,
	SympyMax, SympyMin)
from libsignetsim.model.sbml.KineticLawIdentifier import KineticLawIdentifier
from libsignetsim.settings.Settings import Settings

from sympy import simplify, srepr, diff, zeros, pretty
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs

class KineticLaw(KineticLawIdentifier):

	def __init__(self, model, isFromReaction):

		self.__model = model
		self.__definition = MathFormula(model, MathFormula.MATH_KINETICLAW, isFromReaction)
		KineticLawIdentifier.__init__(self, model, isFromReaction)

	def copy(self, obj, symbols_subs={}, conversion_factors={}, extent_conversion=None, time_conversion=None):

		t_convs = {}
		for var, conversion in list(conversion_factors.items()):
			t_convs.update({var: var/conversion})

		t_formula = unevaluatedSubs(obj.kineticLaw.getDefinition().getInternalMathFormula(), symbols_subs)
		t_formula = unevaluatedSubs(t_formula, t_convs)

		if extent_conversion is not None:
			t_formula *= extent_conversion.getInternalMathFormula()

		if time_conversion is not None:
			t_formula /= time_conversion.getInternalMathFormula()

		self.__definition.setInternalMathFormula(t_formula)

	def readSbml(self, sbml_math,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		self.__definition.readSbml(sbml_math, sbml_level, sbml_version)
		self.__definition.setInternalMathFormula(self.__definition.ensureFloat(self.__definition.getInternalMathFormula()))
		# And the we should look for the Kinetic law Here
		# But since it may take time, we will do it on demand
		# KineticLawIdentifier.findKineticLaw(self)

	def getRawDefinition(self, rawFormula=False):

		formula = self.__definition.getInternalMathFormula()

		if formula is not None:
			if (not rawFormula and len(self.reaction.listOfReactants) > 0
				and self.reaction.listOfReactants[0].getSpecies().isConcentration()):
				comp = self.reaction.listOfReactants[0].getSpecies().getCompartment()

				formula = SympyMul(
					formula,
					SympyPow(
						comp.symbol.getInternalMathFormula(),
						SympyInteger(-1),
						evaluate=False
					),
					evaluate=False
				)

			if not rawFormula:
				subs = {}
				for species in self.__model.listOfSpecies:
					if species.isConcentration():
						subs.update({species.symbol.getInternalMathFormula(rawFormula=True): species.symbol.getInternalMathFormula()})
				formula = unevaluatedSubs(formula, subs)
		return simplify(formula)


	def getDefinition(self, rawFormula=False):
		""" Getting the kinetic's law definition. It follows the sbml specification's type of kinetic law, 
			aka in substance units. rawFormula = True will return that (all rowFormula will work on substances"
			So it rawFormula is false, we must divide by the compartment. We must also switch all concentration 
			symbols with simple species symbols
		"""

		math_formula = MathFormula(self.__model, MathFormula.MATH_KINETICLAW)
		math_formula.setInternalMathFormula(self.getRawDefinition(rawFormula=rawFormula))
		return math_formula

	def setPrettyPrintMathFormula(self, definition, forcedConcentration=False):

		if (forcedConcentration and len(self.reaction.listOfReactants) > 0
			and self.reaction.listOfReactants[0].getSpecies().isConcentration()):
			t_comp = self.reaction.listOfReactants[0].getSpecies().getCompartment()
			t_math_formula = MathFormula(self.__model, MathFormula.MATH_KINETICLAW)
			t_math_formula.setPrettyPrintMathFormula(definition, forcedConcentration)
			self.__definition.setInternalMathFormula(t_math_formula.getInternalMathFormula()*t_comp.symbol.getInternalMathFormula())

		else:
			self.__definition.setPrettyPrintMathFormula(definition, forcedConcentration)

		if self.reaction.value is None:
			self.reaction.value = MathFormula(self.__model)
		self.reaction.value.setInternalMathFormula(self.__definition.getInternalMathFormula())


	def getPrettyPrintMathFormula(self):

		if len(self.reaction.listOfReactants) > 0:
			first_reactant = self.reaction.listOfReactants[0].getSpecies()
			if first_reactant.isConcentration():
				t_comp = first_reactant.getCompartment()
				t_math_formula = MathFormula(self.__model, MathFormula.MATH_KINETICLAW)
				t_math_formula.setInternalMathFormula(
					self.__definition.getInternalMathFormula() / t_comp.symbol.getInternalMathFormula()
				)
				t_math_formula = t_math_formula.getPrettyPrintMathFormula()
			else:
				t_math_formula = self.__definition.getPrettyPrintMathFormula()
		else:
			t_math_formula = self.__definition.getPrettyPrintMathFormula()

		if len(self.reaction.listOfLocalParameters) > 0:
			t_math_formula = t_math_formula.replace("_local_%d_" % (self.reaction.objId), "")

		for species in self.__model.listOfSpecies:
			if species.isConcentration():
				t_math_formula = t_math_formula.replace(("_speciesForcedConcentration_%s_" % (species.symbol.getInternalMathFormula())), species.getSbmlId())

		return t_math_formula


	def getReactantsFormula(self):

		t_reactants = 1
		for reactant in self.reaction.listOfReactants:

			t_reactants *= reactant.getSpecies().symbol.getInternalMathFormula(rawFormula=True)

			if not reactant.stoichiometry.isOne():
				t_reactants *= reactant.stoichiometry.getInternalMathFormula(rawFormula=True)

		return t_reactants


	def getModifiersFormula(self):

		t_modifiers = 1
		for modifier in self.reaction.listOfModifiers:
			t_modifiers *= modifier.getSpecies().symbol.getInternalMathFormula(rawFormula=True)

			if not modifier.stoichiometry.isOne():
				t_modifiers *= modifier.stoichiometry.getInternalMathFormula(rawFormula=True)

		return t_modifiers


	def getProductsFormula(self):

		t_products = 1
		for product in self.reaction.listOfProducts:
			t_products *= product.getSpecies().symbol.getInternalMathFormula(rawFormula=True)

			if not product.stoichiometry.isOne():
				t_products *= product.stoichiometry.getInternalMathFormula(rawFormula=True)

		return t_products


	def setMassAction(self, parameters, reversible):

		t_reactants = self.getReactantsFormula()
		t_modifiers = self.getModifiersFormula()

		t_formula = t_reactants*t_modifiers
		t_formula *= parameters[0].symbol.getInternalMathFormula()

		if reversible:
			t_products = self.getProductsFormula()
			t_formula_back = t_modifiers*t_products
			t_formula_back *= parameters[1].symbol.getInternalMathFormula()

			t_formula -= t_formula_back

		if len(self.reaction.listOfReactants) > 0:
			first_reactant = self.reaction.listOfReactants[0].getSpecies()
			if not first_reactant.hasOnlySubstanceUnits:
				t_formula *= first_reactant.getCompartment().symbol.getInternalMathFormula()

		elif len(self.reaction.listOfModifiers) > 0:
			first_modifier = self.reaction.listOfModifiers[0].getSpecies()
			if not first_modifier.hasOnlySubstanceUnits:
				t_formula *= first_modifier.getCompartment().symbol.getInternalMathFormula()

		self.__definition.setInternalMathFormula(simplify(t_formula))


	def setMichaelis(self, parameters):

		t_reactants = self.getReactantsFormula()
		t_modifiers = self.getModifiersFormula()

		t_kcat = parameters[0].symbol.getInternalMathFormula()
		t_km = parameters[1].symbol.getInternalMathFormula()

		t_formula = t_kcat*t_reactants*t_modifiers/(t_reactants + t_km)

		if len(self.reaction.listOfReactants) > 0:
			first_reactant = self.reaction.listOfReactants[0].getSpecies()
			if not first_reactant.hasOnlySubstanceUnits:
				t_formula *= first_reactant.getCompartment().symbol.getInternalMathFormula()

		elif len(self.reaction.listOfModifiers) > 0:
			first_modifier = self.reaction.listOfModifiers[0].getSpecies()
			if not first_modifier.hasOnlySubstanceUnits:
				t_formula *= first_modifier.getCompartment().symbol.getInternalMathFormula()

		self.__definition.setInternalMathFormula(simplify(t_formula))



	def setHill(self, parameters):

		t_reactants = self.getReactantsFormula()
		t_modifiers = self.getModifiersFormula()

		t_kcat = parameters[0].symbol.getInternalMathFormula()
		t_kd = parameters[1].symbol.getInternalMathFormula()
		t_n = parameters[2].symbol.getInternalMathFormula()

		t_r_pow = t_reactants**t_n
		t_formula = t_kcat*t_modifiers*t_r_pow/(t_r_pow + t_kd)

		if len(self.reaction.listOfReactants) > 0:
			first_reactant = self.reaction.listOfReactants[0].getSpecies()
			if not first_reactant.hasOnlySubstanceUnits:
				t_formula *= first_reactant.getCompartment().symbol.getInternalMathFormula()

		elif len(self.reaction.listOfModifiers) > 0:
			first_modifier = self.reaction.listOfModifiers[0].getSpecies()
			if not first_modifier.hasOnlySubstanceUnits:
				t_formula *= first_modifier.getCompartment().symbol.getInternalMathFormula()

		self.__definition.setInternalMathFormula(simplify(t_formula))

	def getRawVelocities(self, subs={}, include_fast_reaction=True, include_slow_reaction=True):

		definition = self.__definition.getDeveloppedInternalMathFormula()
		if not KineticLawIdentifier.isReversible(self, definition):
			res = zeros(1)
			if (self.reaction.fast and include_fast_reaction) or (not self.reaction.fast and include_slow_reaction):
				res[0] += unevaluatedSubs(definition, subs)
			return res

		else:
			res_front = zeros(1)
			res_backward = zeros(1)
			if (self.reaction.fast and include_fast_reaction) or (not self.reaction.fast and include_slow_reaction):
				(definition_front, definition_back) = KineticLawIdentifier.getReversibleRates(self, definition)
				res_front[0] += unevaluatedSubs(definition_front, subs)
				res_backward[0] += unevaluatedSubs(definition_back, subs)

			return res_front.col_join(res_backward)

	def renameSbmlId(self, old_sbml_id, new_sbml_id):

		self.__definition.renameSbmlId(
			"_local_%d_%s" % (self.reaction.objId, old_sbml_id),
			"_local_%d_%s" % (self.reaction.objId, new_sbml_id)
		)
		self.__definition.renameSbmlId(old_sbml_id, new_sbml_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function
from __future__ import absolute_import



from libsignetsim.sedml.SedmlException import SedmlMathException

import libsbml
# token: cn , ci , csymbol , sep
from libsedml import AST_NAME, AST_NAME_TIME
from .sympy_shortcuts import SympyInteger, SympyFloat, SympySymbol
# qualifiers: degree , bvar , logbase

# general : apply , piecewise , piece , otherwise , lambda
from libsedml import AST_FUNCTION_PIECEWISE, AST_LAMBDA
from .sympy_shortcuts import SympyPiecewise, SympyITE, SympyLambda

# relational operators: eq , neq , gt , lt , geq , leq
from libsedml import (
	AST_RELATIONAL_EQ, AST_RELATIONAL_NEQ, AST_RELATIONAL_GT, AST_RELATIONAL_LT, AST_RELATIONAL_GEQ, AST_RELATIONAL_LEQ
)
from .sympy_shortcuts import (
	SympyEqual, SympyUnequal, SympyStrictGreaterThan, SympyStrictLessThan, SympyGreaterThan, SympyLessThan
)

# arithmetic operators: plus , minus , times , divide , power , root , abs , exp , ln , log , floor , ceiling ,
from libsedml import (
	AST_PLUS, AST_MINUS, AST_TIMES, AST_DIVIDE, AST_POWER, AST_FUNCTION_POWER, AST_FUNCTION_ROOT, AST_FUNCTION_ABS, AST_FUNCTION_EXP,
	AST_FUNCTION_LN, AST_FUNCTION_LOG, AST_FUNCTION_FLOOR, AST_FUNCTION_CEILING
)
from .sympy_shortcuts import (SympyAdd, SympyMul, SympyPow, SympyAbs, SympyExp, SympyLog, SympyFloor, SympyCeiling)


# factorial
from libsedml import AST_FUNCTION_FACTORIAL
from .sympy_shortcuts import SympyFactorial

# logical operators: and , or , xor , not
from libsedml import AST_LOGICAL_AND, AST_LOGICAL_OR, AST_LOGICAL_XOR, AST_LOGICAL_NOT
from .sympy_shortcuts import SympyAnd, SympyOr, SympyXor, SympyNot

# trigonometric operators: sin , cos , tan , sec , csc , cot
from libsedml import (
	AST_FUNCTION_SIN, AST_FUNCTION_COS, AST_FUNCTION_TAN, AST_FUNCTION_SEC, AST_FUNCTION_CSC, AST_FUNCTION_COT
)
from .sympy_shortcuts import (SympySin, SympyCos, SympyTan, SympySec, SympyCsc, SympyCot)


# trigonometric operators: sinh , cosh , tanh , sech , csch , coth ,
from libsedml import (
	AST_FUNCTION_SINH, AST_FUNCTION_COSH, AST_FUNCTION_TANH, AST_FUNCTION_SECH, AST_FUNCTION_CSCH, AST_FUNCTION_COTH
)
from .sympy_shortcuts import (SympySinh, SympyCosh, SympyTanh, SympyCoth)

# trigonometric operators: arcsin , arccos , arctan , arcsec , arccsc , arccot
from libsedml import (
	AST_FUNCTION_ARCSIN, AST_FUNCTION_ARCCOS, AST_FUNCTION_ARCTAN, AST_FUNCTION_ARCSEC, AST_FUNCTION_ARCCSC,
	AST_FUNCTION_ARCCOT
)
from .sympy_shortcuts import (SympyAsin, SympyAcos, SympyAtan, SympyAsec, SympyAcsc, SympyAcot)

# trigonometric operators: arcsinh , arccosh , arctanh , arcsech , arccsch , arccoth
from libsedml import (
	AST_FUNCTION_ARCSINH, AST_FUNCTION_ARCCOSH, AST_FUNCTION_ARCTANH, AST_FUNCTION_ARCSECH,
	AST_FUNCTION_ARCCSCH, AST_FUNCTION_ARCCOTH
)
from .sympy_shortcuts import (SympyAsinh, SympyAcosh, SympyAtanh, SympyAcoth)

# aggregate functions : min, max, sum, product
from .sympy_shortcuts import SympyFunction

# constants: true , false , notanumber , pi , infinity , exponentiale
# TODO : Infinity, NaN
from libsedml import (AST_CONSTANT_TRUE, AST_CONSTANT_FALSE, AST_CONSTANT_PI, AST_CONSTANT_E)
from .sympy_shortcuts import (SympyTrue, SympyFalse, SympyPi, SympyE, SympyInf, SympyNan)

from libsedml import parseFormula, formulaToString
from six.moves import reload_module
reload_module(libsbml)


class SedmlMathReader(object):
	""" Class for handling math formulaes """

	def __init__(self, document):
		""" Constructor """

		self.__document = document

	def translateForInternal(self, tree):

		""" Translate an SBML Tree in a Sympy Tree """

		if tree.isInfinity():
			return SympyInf

		elif tree.isNegInfinity():
			return SympyMul(SympyInteger(-1), SympyInf)

		elif tree.isNaN():
			return SympyNan

		elif tree.getType() == AST_NAME:
			return SympySymbol(tree.getName())

		elif tree.getType() == AST_NAME_TIME:
			return SympySymbol("time")

		elif tree.isNumber():

			if tree.isInteger():
				return SympyInteger(tree.getInteger())

			elif tree.isReal():
				return SympyFloat(tree.getReal())

			else:
				raise SedmlMathException("SedMathReader : Unknown type of number")

		elif tree.isConstant():
			if tree.getType() == AST_CONSTANT_E:
				return SympyE

			elif tree.getType() == AST_CONSTANT_FALSE:
				return SympyFalse

			elif tree.getType() == AST_CONSTANT_TRUE:
				return SympyTrue

			elif tree.getType() == AST_CONSTANT_PI:
				return SympyPi

			else:
				raise SedmlMathException("SedmlMathReader : Unknown constant")

		elif tree.isOperator():

			if tree.getType() == AST_PLUS:

				if tree.getNumChildren() == 0:
					return SympyInteger(0)

				else:
					t_children = []
					for i_arg in range(tree.getNumChildren()):
						t_children.append(self.translateForInternal(tree.getChild(i_arg)))
					return SympyAdd(*t_children, evaluate=False)

			elif tree.getType() == AST_MINUS:

				if tree.getNumChildren() == 2:
					return SympyAdd(
								self.translateForInternal(tree.getChild(0)),
								SympyMul(
									SympyInteger(-1),
									self.translateForInternal(tree.getChild(1)),
									evaluate=False),
								evaluate=False)

				elif tree.getNumChildren() == 1:
					return SympyMul(
								SympyInteger(-1),
								self.translateForInternal(tree.getChild(0)),
								evaluate=False)

				elif tree.getNumChildren() == 0:
					return SympyInteger(0)

				else:
					t_tree = SympyAdd(
								self.translateForInternal(tree.getChild(0)),
								SympyMul(
									SympyInteger(-1),
									self.translateForInternal(tree.getChild(1)), evaluate=False),
								evaluate=False)

					for i_arg in range(2, tree.getNumChildren()):
						t_tree = SympyAdd(
									t_tree,
									SympyMul(
										SympyInteger(-1),
										self.translateForInternal(tree.getChild(i_arg)), evaluate=False),
									evaluate=False)

					return t_tree

			elif tree.getType() == AST_TIMES:

				if tree.getNumChildren() == 0:
					return SympyInteger(1)
				else:
					t_children = []

					for i_arg in range(tree.getNumChildren()):
						t_children.append(self.translateForInternal(tree.getChild(i_arg)))

					return SympyMul(*t_children, evaluate=False)

			elif tree.getType() == AST_DIVIDE:
				return SympyMul(
								self.translateForInternal(tree.getChild(0)),
								SympyPow(self.translateForInternal(tree.getChild(1)), SympyInteger(-1)))

			elif tree.getType() == AST_POWER or tree.getType() == AST_FUNCTION_POWER:
				t_x = self.translateForInternal(tree.getChild(0))
				t_n = self.translateForInternal(tree.getChild(1))
				return SympyPow(t_x, t_n, evaluate=False)

			else:
				raise SedmlMathException("SedmlMathReader : Unknown operator")

		elif tree.isFunction():

				# AST_FUNCTION_ABS
			if tree.getType() == AST_FUNCTION_ABS:
				return SympyAbs(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCCOS
			elif tree.getType() == AST_FUNCTION_ARCCOS:
				return SympyAcos(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCCOSH
			elif tree.getType() == AST_FUNCTION_ARCCOSH:
				return SympyAcosh(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCCOT
			elif tree.getType() == AST_FUNCTION_ARCCOT:
				return SympyAcot(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCCOTH
			elif tree.getType() == AST_FUNCTION_ARCCOTH:
				return SympyAcoth(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCCSC
			elif tree.getType() == AST_FUNCTION_ARCCSC:
				return SympyAcsc(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCCSCH
			elif tree.getType() == AST_FUNCTION_ARCCSCH:
				return SympyAsinh(
						SympyPow(self.translateForInternal(tree.getChild(0)), SympyInteger(-1), evaluate=False),
						evaluate=False)

				# AST_FUNCTION_ARCSEC
			elif tree.getType() == AST_FUNCTION_ARCSEC:
				return SympyAsec(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCSECH
			elif tree.getType() == AST_FUNCTION_ARCSECH:
				return SympyAcosh(
						SympyPow(self.translateForInternal(tree.getChild(0)), SympyInteger(-1), evaluate=False),
						evaluate=False)

				# AST_FUNCTION_ARCSIN
			elif tree.getType() == AST_FUNCTION_ARCSIN:
				return SympyAsin(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCSINH
			elif tree.getType() == AST_FUNCTION_ARCSINH:
				return SympyAsinh(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCTAN
			elif tree.getType() == AST_FUNCTION_ARCTAN:
				return SympyAtan(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_ARCTANH
			elif tree.getType() == AST_FUNCTION_ARCTANH:
				return SympyAtanh(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_CEILING
			elif tree.getType() == AST_FUNCTION_CEILING:
				return SympyCeiling(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_COS
			elif tree.getType() == AST_FUNCTION_COS:
				return SympyCos(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_COSH
			elif tree.getType() == AST_FUNCTION_COSH:
				return SympyCosh(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_COT
			elif tree.getType() == AST_FUNCTION_COT:
				return SympyCot(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_COTH
			elif tree.getType() == AST_FUNCTION_COTH:
				return SympyCoth(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_CSC
			elif tree.getType() == AST_FUNCTION_CSC:
				return SympyCsc(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_CSCH
			elif tree.getType() == AST_FUNCTION_CSCH:
				# return SympyCsch(self.translateForInternal(tree.getChild(0)), evaluate=False)
				return SympyPow(
						SympySinh(self.translateForInternal(tree.getChild(0)), evaluate=False),
						SympyInteger(-1), evaluate=False)

				# AST_FUNCTION_EXP
			elif tree.getType() == AST_FUNCTION_EXP:
				return SympyExp(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_FACTORIAL
			elif tree.getType() == AST_FUNCTION_FACTORIAL:
				return SympyFactorial(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_FLOOR
			elif tree.getType() == AST_FUNCTION_FLOOR:
				return SympyFloor(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_LN
			elif tree.getType() == AST_FUNCTION_LN:
				return SympyLog(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_LOG
			elif tree.getType() == AST_FUNCTION_LOG:
				t_n = self.translateForInternal(tree.getChild(0))
				t_x = self.translateForInternal(tree.getChild(1))

				return SympyLog(t_x, t_n, evaluate=False)

				# AST_FUNCTION_PIECEWISE
			elif tree.getType() == AST_FUNCTION_PIECEWISE:

				i_arg = 0
				t_pieces = []
				value_piecewise = False
				while i_arg < tree.getNumChildren():

					if (i_arg+1) < tree.getNumChildren():
						t_value = self.translateForInternal(tree.getChild(i_arg))
						t_condition = self.translateForInternal(tree.getChild(i_arg+1))

						if isinstance(t_value, bool):
							value_piecewise = True

						t_pieces.append((t_value, t_condition))
						i_arg += 2
					else:
						t_value = self.translateForInternal(tree.getChild(i_arg))
						if isinstance(t_value, bool):
							value_piecewise = True
						t_pieces.append((t_value, True))
						i_arg += 1

				if not value_piecewise:
					return SympyPiecewise(*t_pieces, evaluate=False)
				else:
					return SympyITE(t_pieces[0][1], t_pieces[0][0], t_pieces[1][0])

				# AST_FUNCTION_POWER
			elif tree.getType() == AST_POWER or tree.getType() == AST_FUNCTION_POWER:
				t_x = self.translateForInternal(tree.getChild(0))
				t_n = self.translateForInternal(tree.getChild(1))
				return SympyPow(t_x, t_n, evaluate=False)

				# AST_FUNCTION_ROOT
			elif tree.getType() == AST_FUNCTION_ROOT:
				return SympyPow(
						self.translateForInternal(tree.getChild(1)),
						SympyPow(
							self.translateForInternal(tree.getChild(0)),
							SympyInteger(-1), evaluate=False),
						evaluate=False)

				# AST_FUNCTION_SEC
			elif tree.getType() == AST_FUNCTION_SEC:
				return SympySec(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_SECH
			elif tree.getType() == AST_FUNCTION_SECH:
				return SympyPow(
							SympyCosh(self.translateForInternal(tree.getChild(0)), evaluate=False),
							SympyInteger(-1), evaluate=False)

				# AST_FUNCTION_SIN
			elif tree.getType() == AST_FUNCTION_SIN:
				return SympySin(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_SINH
			elif tree.getType() == AST_FUNCTION_SINH:
				return SympySinh(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_TAN
			elif tree.getType() == AST_FUNCTION_TAN:
				return SympyTan(self.translateForInternal(tree.getChild(0)), evaluate=False)

				# AST_FUNCTION_TANH
			elif tree.getType() == AST_FUNCTION_TANH:
				return SympyTanh(self.translateForInternal(tree.getChild(0)), evaluate=False)

			else:
				str_tree = formulaToString(tree)
				if str_tree.startswith("min("):
					return SympyFunction("min")(*([self.translateForInternal(tree.getChild(0))]))

				elif str_tree.startswith("max("):
					return SympyFunction("max")(*([self.translateForInternal(tree.getChild(0))]))

				elif str_tree.startswith("sum("):
					return SympyFunction("sum")(*([self.translateForInternal(tree.getChild(0))]))

				elif str_tree.startswith("product("):
					return SympyFunction("product")(*([self.translateForInternal(tree.getChild(0))]))

				print(tree.getType() == AST_POWER)
				print(tree.getType() == AST_FUNCTION_POWER)
				raise SedmlMathException("SedmlMathReader : Unknown function %s" % formulaToString(tree))

		elif tree.getType() == AST_LAMBDA:

			t_args = []
			for param in range(0, tree.getNumChildren()-1):
				t_args.append(self.translateForInternal(tree.getChild(param)))

			t_def = self.translateForInternal(tree.getChild(tree.getNumChildren()-1))

			return SympyLambda(tuple(t_args), t_def)

		elif tree.isRelational():

			if tree.getType() == AST_RELATIONAL_EQ:
				t_res = SympyEqual(
						self.translateForInternal(tree.getChild(0)),
						self.translateForInternal(tree.getChild(1)), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a==b && b==c && c==d, which should be equal to a==b==c==d
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyEqual(
								self.translateForInternal(tree.getChild(child-1)),
								self.translateForInternal(tree.getChild(child)), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res

			elif tree.getType() == AST_RELATIONAL_NEQ:
				t_res = SympyUnequal(
						self.translateForInternal(tree.getChild(0)),
						self.translateForInternal(tree.getChild(1)), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a!=b && b!=c && c!=d, which should be equal to a!=b!=c!=d
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyUnequal(
								self.translateForInternal(tree.getChild(child-1)),
								self.translateForInternal(tree.getChild(child)), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res

			elif tree.getType() == AST_RELATIONAL_GT:
				t_res = SympyStrictGreaterThan(
						self.translateForInternal(tree.getChild(0)),
						self.translateForInternal(tree.getChild(1)), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a>b && b>c, which should be equal to a>b>c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyStrictGreaterThan(
								self.translateForInternal(tree.getChild(child-1)),
								self.translateForInternal(tree.getChild(child)), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res

			elif tree.getType() == AST_RELATIONAL_LT:
				t_res = SympyStrictLessThan(
						self.translateForInternal(tree.getChild(0)),
						self.translateForInternal(tree.getChild(1)), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a<b && b<c, which should be equal to a<b<c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyStrictLessThan(
								self.translateForInternal(tree.getChild(child-1)),
								self.translateForInternal(tree.getChild(child)), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res

			elif tree.getType() == AST_RELATIONAL_GEQ:
				t_res = SympyGreaterThan(
						self.translateForInternal(tree.getChild(0)),
						self.translateForInternal(tree.getChild(1)), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a>=b && b>=c, which should be equal to a>=b>=c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyGreaterThan(
								self.translateForInternal(tree.getChild(child-1)),
								self.translateForInternal(tree.getChild(child)), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res

			elif tree.getType() == AST_RELATIONAL_LEQ:
				t_res = SympyLessThan(
						self.translateForInternal(tree.getChild(0)),
						self.translateForInternal(tree.getChild(1)), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a<=b && b<=c, which should be equal to a<=b<=c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyLessThan(
								self.translateForInternal(tree.getChild(child-1)),
								self.translateForInternal(tree.getChild(child)), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res

			else:
				raise SedmlMathException("SedmlMathReader : Unknown relational operator")

		elif tree.isLogical():

			if tree.getType() == AST_LOGICAL_AND:
				t_children = []
				for child in range(0, tree.getNumChildren()):
					t_children.append(self.translateForInternal(tree.getChild(child)))

				return SympyAnd(*t_children, evaluate=False)

			elif tree.getType() == AST_LOGICAL_OR:
				t_children = []
				for child in range(0, tree.getNumChildren()):
					t_children.append(self.translateForInternal(tree.getChild(child)))

				return SympyOr(*t_children, evaluate=False)

			elif tree.getType() == AST_LOGICAL_XOR:
				t_children = []
				for child in range(0, tree.getNumChildren()):
					t_children.append(self.translateForInternal(tree.getChild(child)))

				return SympyXor(*t_children, evaluate=False)

			elif tree.getType() == AST_LOGICAL_NOT:
				return SympyNot(self.translateForInternal(tree.getChild(0)), evaluate=False)

			else:
				raise SedmlMathException("SedmlMathReader : Unknown logical operator")

		else:
			raise SedmlMathException("SedmlMathReader : Unknown mathematical term : %s" % parseFormula(tree))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.model.sbml.HasId import HasId
from libsignetsim.settings.Settings import Settings

from libsbml import UNIT_KIND_AMPERE, UNIT_KIND_AVOGADRO, UNIT_KIND_BECQUEREL,\
	UNIT_KIND_CANDELA, UNIT_KIND_COULOMB, UNIT_KIND_DIMENSIONLESS,\
	UNIT_KIND_FARAD, UNIT_KIND_GRAM, UNIT_KIND_GRAY, UNIT_KIND_HENRY,\
	UNIT_KIND_HERTZ, UNIT_KIND_ITEM, UNIT_KIND_JOULE, UNIT_KIND_KATAL,\
	UNIT_KIND_KELVIN, UNIT_KIND_KILOGRAM, UNIT_KIND_LITRE, UNIT_KIND_LUMEN,\
	UNIT_KIND_LUX, UNIT_KIND_METRE, UNIT_KIND_MOLE, UNIT_KIND_NEWTON,\
	UNIT_KIND_OHM, UNIT_KIND_PASCAL, UNIT_KIND_RADIAN, UNIT_KIND_SECOND,\
	UNIT_KIND_SIEMENS, UNIT_KIND_SIEVERT, UNIT_KIND_STERADIAN, UNIT_KIND_TESLA,\
	UNIT_KIND_VOLT, UNIT_KIND_WATT, UNIT_KIND_WEBER, UNIT_KIND_INVALID


class Unit(object):

	unit_id = {UNIT_KIND_AMPERE: "ampere",
						UNIT_KIND_AVOGADRO: "avogadro",
						UNIT_KIND_BECQUEREL: "becquerel",
						UNIT_KIND_CANDELA: "candela",
						UNIT_KIND_COULOMB: "coulomb",
						UNIT_KIND_DIMENSIONLESS: "dimensionless",
						UNIT_KIND_FARAD: "farad",
						UNIT_KIND_GRAM: "gram",
						UNIT_KIND_GRAY: "gray",
						UNIT_KIND_HENRY: "henry",
						UNIT_KIND_HERTZ: "hertz",
						UNIT_KIND_ITEM: "thing",
						UNIT_KIND_JOULE: "joule",
						UNIT_KIND_KATAL: "katal",
						UNIT_KIND_KELVIN: "kelvin",
						UNIT_KIND_KILOGRAM: "kilogram",
						UNIT_KIND_LITRE: "litre",
						UNIT_KIND_LUMEN: "lumen",
						UNIT_KIND_LUX: "lux",
						UNIT_KIND_METRE: "metre",
						UNIT_KIND_MOLE: "mole",
						UNIT_KIND_NEWTON: "newton",
						UNIT_KIND_OHM: "ohm",
						UNIT_KIND_PASCAL: "pascal",
						UNIT_KIND_RADIAN: "radian",
						UNIT_KIND_SECOND: "second",
						UNIT_KIND_SIEMENS: "siemens",
						UNIT_KIND_SIEVERT: "sievert",
						UNIT_KIND_STERADIAN: "steradian",
						UNIT_KIND_TESLA: "tesla",
						UNIT_KIND_VOLT: "volt",
						UNIT_KIND_WATT: "watt",
						UNIT_KIND_WEBER: "weber",
						UNIT_KIND_INVALID: "invalid"
	}


	def __init__(self, model=None, objId=None):

		self.__model = model
		self.objId = objId
		self.kind = UNIT_KIND_MOLE
		self.exponent = 1
		self.scale = 0
		self.multiplier = 1


	def new(self, kind = UNIT_KIND_MOLE, exponent=1, scale=1, multiplier=1):

		self.kind = kind
		self.exponent = exponent
		self.scale = scale
		self.multiplier = multiplier


	def copy(self, obj, prefix="", shift=0):

		self.kind = obj.kind
		self.exponent = obj.exponent
		self.scale = obj.scale
		self.multiplier = obj.multiplier


	def isEqual(self, other_unit):

		return (self.exponent == other_unit.exponent
			and self.multiplier*(10**self.scale) == other_unit.multiplier*(10**other_unit.scale))


	def readSbml(self, unit, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		self.kind = int(unit.getKind())

		if unit.isSetExponent():
			self.exponent = int(unit.getExponent())

		if unit.isSetScale():
			self.scale = int(unit.getScale())

		if unit.isSetMultiplier():
			self.multiplier = float(unit.getMultiplier())


	def writeSbml(self, unitDefinition, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		t_unit = unitDefinition.createUnit()

		t_unit.setKind(self.kind)

		if self.exponent != 1 or sbml_level == 3:
			t_unit.setExponent(self.exponent)

		if self.scale != 0 or sbml_level == 3:
			t_unit.setScale(self.scale)

		if self.multiplier != 1 or sbml_level == 3:
			t_unit.setMultiplier(self.multiplier)


	def setKind(self, kind):
		self.kind = kind


	def setExponent(self, exponent):
		self.exponent = exponent


	def setScale(self, scale):
		self.scale = scale

	def __str__(self):
		t_str = ""
		if self.multiplier != 1:
			t_str += "(%g." % self.multiplier

		if self.scale == -3:
			t_str += "m"
		elif self.scale == -6:
			t_str += "u"
		elif self.scale == -9:
			t_str += "n"
		elif self.scale == +3:
			t_str += "k"
		elif self.scale == +6:
			t_str += "M"
		elif self.scale == +9:
			t_str += "G"

		t_str += self.unit_id[self.kind] + ("^" + str(self.exponent) if self.exponent != 1 else "")

		if self.multiplier != 1:
			t_str += ")"
		return t_str

	def getKind(self):
		return self.kind

	def getKindName(self):
		return self.unit_id[self.kind]

	def getExponent(self):
		return self.exponent

	def getScale(self):
		return self.scale

	def getMultiplier(self):
		return self.multiplier



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.model.sbml.container.ListOfSpecies import ListOfSpecies
from libsignetsim.model.sbml.container.ListOfParameters import ListOfParameters
from libsignetsim.model.sbml.container.ListOfReactions import ListOfReactions
from libsignetsim.model.sbml.container.ListOfCompartments import ListOfCompartments
from libsignetsim.model.sbml.container.ListOfFunctionDefinitions import ListOfFunctionDefinitions
from libsignetsim.model.sbml.container.ListOfUnitDefinitions import ListOfUnitDefinitions
from libsignetsim.model.sbml.container.ListOfRules import ListOfRules
from libsignetsim.model.sbml.container.ListOfEvents import ListOfEvents
from libsignetsim.model.sbml.container.ListOfConstraints import ListOfConstraints
from libsignetsim.model.sbml.container.ListOfInitialAssignments import ListOfInitialAssignments
from libsignetsim.model.sbml.container.ListOfSbmlObjects import ListOfSbmlObjects
from libsignetsim.model.sbml.container.ListOfSubmodels import ListOfSubmodels
from libsignetsim.model.sbml.container.ListOfPorts import ListOfPorts

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.SbmlModelAnnotation import SbmlModelAnnotation
from libsignetsim.model.sbml.HasId import HasId

from os.path import isfile
from libsbml import SBMLReader, SBMLDocument, formulaToString,\
					formulaToL3String, writeSBMLToFile,\
					XMLFileUnreadable, XMLFileOperationError, \
					LIBSBML_CAT_UNITS_CONSISTENCY, LIBSBML_SEV_INFO, \
					LIBSBML_SEV_WARNING, SBMLExtensionRegistry

from time import time

class ModelUnits(object):
	""" Sbml model units class """


	def __init__ (self):
		""" Constructor of model class """


		self.timeUnits = None
		self.substanceUnits = None
		self.concentrationUnits = None
		self.compartmentUnits = None
		self.extentUnits = None


	def setDefaultUnits(self):

		self.substanceUnits = self.listOfUnitDefinitions.new()
		self.substanceUnits.defaultAmountUnit()

		self.extentUnits = self.substanceUnits

		self.timeUnits = self.listOfUnitDefinitions.new()
		self.timeUnits.defaultTimeUnits()

		self.concentrationUnits = self.listOfUnitDefinitions.new()
		self.concentrationUnits.defaultConcentrationUnit()

		self.compartmentUnits = self.listOfUnitDefinitions.new()
		self.compartmentUnits.defaultCompartmentUnits()

		# self.defaultCompartment.setUnits(self.compartmentUnits)

	def readSbml(self, sbmlModel,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion, parent_doc=None):

		# We need to load the units before reading the model's default units
		if self.sbmlLevel >= 3:
			if sbmlModel.isSetTimeUnits():
				self.timeUnits = self.listOfUnitDefinitions.getBySbmlId(
												sbmlModel.getTimeUnits())

			if sbmlModel.isSetSubstanceUnits():
				self.substanceUnits = self.listOfUnitDefinitions.getBySbmlId(
												sbmlModel.getSubstanceUnits())


			if sbmlModel.isSetExtentUnits():
				self.extentUnits = self.listOfUnitDefinitions.getBySbmlId(
												sbmlModel.getExtentUnits())

		else:
			if self.listOfUnitDefinitions.containsSbmlId("time"):
				self.timeUnits = self.listOfUnitDefinitions.getBySbmlId("time")

			if self.listOfUnitDefinitions.containsSbmlId("substance"):
				self.substanceUnits = self.listOfUnitDefinitions.getBySbmlId("substance")



	def writeSbml(self, sbmlModel,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):


		if self.sbmlLevel >= 3:
			if self.timeUnits is not None:
				sbmlModel.setTimeUnits(self.timeUnits.getSbmlId())
			if self.substanceUnits is not None:
				sbmlModel.setSubstanceUnits(self.substanceUnits.getSbmlId())
			if self.extentUnits is not None:
				sbmlModel.setExtentUnits(self.extentUnits.getSbmlId())


	def getTimeUnits(self):
		return self.timeUnits

	def setTimeUnits(self, unit_sbml_id):

		if unit_sbml_id is None:
			self.timeUnits = None

		elif self.sbmlLevel == 3:
			self.timeUnits = self.listOfUnitDefinitions.getBySbmlId(unit_sbml_id)

		elif self.sbmlLevel == 2:

			# First we remove the old one if it exists
			if self.listOfUnitDefinitions.containsSbmlId("time"):

				t_unit_def = self.listOfUnitDefinitions.getBySbmlId("time")
				i=0
				while self.listOfUnitDefinitions.containsSbmlId("time_%d" % i):
					i += 1

				t_unit_def.sbmlId = "time_%d" % i

			t_unit_def = self.listOfUnitDefinitions.getBySbmlId(unit_sbml_id)
			t_unit_def.sbmlId = "time"
			self.timeUnits = t_unit_def


	def getSubstanceUnits(self):
		return self.substanceUnits

	def setSubstanceUnits(self, unit_sbml_id):

		if unit_sbml_id is None:
			self.substanceUnits = None

		elif self.sbmlLevel == 3:
			self.substanceUnits = self.listOfUnitDefinitions.getBySbmlId(unit_sbml_id)

		elif self.sbmlLevel == 2:

			# First we remove the old one if it exists
			if self.listOfUnitDefinitions.containsSbmlId("substance"):

				t_unit_def = self.listOfUnitDefinitions.getBySbmlId("substance")
				i=0
				while self.listOfUnitDefinitions.containsSbmlId("substance_%d" % i):
					i += 1

				t_unit_def.sbmlId = "substance_%d" % i

			t_unit_def = self.listOfUnitDefinitions.getBySbmlId(unit_sbml_id)
			t_unit_def.sbmlId = "substance"
			self.substanceUnits = t_unit_def

	def getExtentUnits(self):
		return self.extentUnits

	def setExtentUnits(self, unit_sbml_id):
		if unit_sbml_id is None:
			self.extentUnits = None

		else:
			self.extentUnits = self.listOfUnitDefinitions.getBySbmlId(unit_sbml_id)


	def getCompartmentUnits(self):
		return self.compartmentUnits


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.Variable import Variable
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.sbml.EventAssignment import EventAssignment
from libsignetsim.model.sbml.EventTrigger import EventTrigger
from libsignetsim.model.sbml.EventDelay import EventDelay
from libsignetsim.model.sbml.EventPriority import EventPriority
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.ModelException import InvalidXPath


class Event(Variable, SbmlObject, HasParentObj):

	""" Events definition """

	def __init__ (self, model, parent_obj, obj_id, math_only=False):

		self.__model = model
		self.objId = obj_id

		Variable.__init__(self, model, Variable.EVENT)
		HasParentObj.__init__(self, parent_obj)

		self.trigger = EventTrigger(model, math_only=math_only)
		self.listOfEventAssignments = []

		self.delay = None
		self.priority = None
		self.useValuesFromTriggerTime = True

		# For math submodels, where objects are not sbml objects
		self.mathOnly = math_only
		if not self.mathOnly:
			SbmlObject.__init__(self, model)

	def new(self, name=None):

		Variable.new(self, name, Variable.EVENT)
		SbmlObject.new(self)

	def readSbml(self, sbml_event,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads an event definition from a sbml file """

		SbmlObject.readSbml(self, sbml_event, sbml_level, sbml_version)

		if sbml_event.getTrigger() is not None:
			self.trigger.readSbml(sbml_event.getTrigger(), sbml_level, sbml_version)

		Variable.readSbml(self, sbml_event, sbml_level, sbml_version)

		if sbml_event.isSetDelay():
			self.delay = EventDelay(self.__model)
			self.delay.readSbml(sbml_event.getDelay(), sbml_level, sbml_version)

		if sbml_event.isSetPriority():
			self.priority = EventPriority(self.__model)
			self.priority.readSbml(sbml_event.getPriority(), sbml_level, sbml_version)

		if sbml_event.isSetUseValuesFromTriggerTime():
			self.useValuesFromTriggerTime = sbml_event.getUseValuesFromTriggerTime()
		else:
			self.useValuesFromTriggerTime = True

		self.listOfEventAssignments = []
		for event_assignment in sbml_event.getListOfEventAssignments():

			t_event_assignment = EventAssignment(self.__model, len(self.listOfEventAssignments), self)
			t_event_assignment.readSbml(event_assignment, sbml_level, sbml_version)
			self.listOfEventAssignments.append(t_event_assignment)

	def readSbmlVariable(self, variable, sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		# variable id
		self.symbol.readSbml(variable.getId(), sbml_level, sbml_version)

		if self.trigger is not None:
			self.value = MathFormula(self.__model)
			self.value.setInternalMathFormula(self.trigger.getInternalMathFormula())
			self.constant = False
		else:
			self.constant = True

	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes an event definition to a sbml file """

		sbml_event = sbml_model.createEvent()

		Variable.writeSbml(self, sbml_event, sbml_level, sbml_version)
		SbmlObject.writeSbml(self, sbml_event, sbml_level, sbml_version)

		if self.trigger is not None:
			self.trigger.writeSbml(sbml_event, sbml_level, sbml_version)

		if self.delay is not None:
			self.delay.writeSbml(sbml_event, sbml_level, sbml_version)

		if self.priority is not None:
			self.priority.writeSbml(sbml_event, sbml_level, sbml_version)

		for event_assignment in self.listOfEventAssignments:
			event_assignment.writeSbml(sbml_event, sbml_level, sbml_version)

		if sbml_level == 2 and self.useValuesFromTriggerTime == True:
			pass
		else:
			sbml_event.setUseValuesFromTriggerTime(self.useValuesFromTriggerTime)

	def writeSbmlVariable(self, sbml_var, sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		pass

	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={},
				conversion_factors={}, time_conversion=None):

		Variable.copy(self, obj, sids_subs=sids_subs, symbols_subs=symbols_subs)

		# if not self.mathOnly:
		SbmlObject.copy(self, obj)

		self.trigger.copy(obj.trigger, symbols_subs=symbols_subs, conversion_factors=conversion_factors)

		if obj.delay is not None and obj.delay not in deletions:
			self.delay = EventDelay(self.__model)
			self.delay.copy(obj.delay, symbols_subs=symbols_subs, conversion_factors=conversion_factors, time_conversion=time_conversion)

		if obj.priority is not None and obj.priority not in deletions:
			self.priority = EventPriority(self.__model)
			self.priority.copy(obj.priority, symbols_subs=symbols_subs, conversion_factors=conversion_factors)

		for event_assignment in obj.listOfEventAssignments:
			if event_assignment not in deletions:
				t_event_assignment = EventAssignment(self.__model, len(self.listOfEventAssignments), self)
				t_event_assignment.copy(
					event_assignment,
					sids_subs=sids_subs,
					symbols_subs=symbols_subs,
					conversion_factors=conversion_factors,
					time_conversion=time_conversion
				)
				self.listOfEventAssignments.append(t_event_assignment)

		self.useValuesFromTriggerTime = obj.useValuesFromTriggerTime

		self.value = MathFormula(self.__model)
		self.value.setInternalMathFormula(self.trigger.getInternalMathFormula())
		self.constant = obj.constant

	def copySubmodel(self, obj):

		self.trigger.copySubmodel(obj.trigger)

		if obj.delay is not None:
			self.delay = EventDelay(self.__model, math_only=self.mathOnly)
			self.delay.copySubmodel(obj.delay)

		if obj.priority is not None:
			self.priority = EventPriority(self.__model, math_only=self.mathOnly)
			self.priority.copySubmodel(obj.priority)

		for event_assignment in obj.listOfEventAssignments:
				t_event_assignment = EventAssignment(self.__model, event_assignment.objId, self, math_only=self.mathOnly)
				t_event_assignment.copySubmodel(event_assignment)
				self.listOfEventAssignments.append(t_event_assignment)
		self.useValuesFromTriggerTime = obj.useValuesFromTriggerTime

	def setTrigger(self, trigger):

		if self.trigger is None:
			self.trigger = EventTrigger(self.__model)

		self.trigger.setPrettyPrintMathFormula(trigger)

	def getTrigger(self):

		if self.trigger is not None:
			return self.trigger.getPrettyPrintMathFormula()

	def getTriggerMath(self):

		return self.trigger

	def getTriggerInitialValue(self):

		return self.trigger.initialValue

	def setTriggerInitialValue(self, value):

		self.trigger.initialValue = value

	def getUseValuesFromTriggerTime(self):

		return self.useValuesFromTriggerTime

	def setUseValuesFromTriggerTime(self, value):

		self.useValuesFromTriggerTime = value

	def isTriggerPersistent(self):

		return self.trigger.isPersistent

	def setTriggerPersistent(self, value):

		self.trigger.isPersistent = value


	def setDelay(self, delay):

		if delay is not None:

			if self.delay is None:
				self.delay = MathFormula(self.__model)
			self.delay.setPrettyPrintMathFormula(delay)

		else:
			self.delay = delay

	def getDelay(self):

		if self.delay is not None:
			return self.delay.getPrettyPrintMathFormula()

	def getDelayMath(self):

		return self.delay

	def setPriority(self, priority):

		if priority is not None:

			if self.priority is None:
				self.priority = MathFormula(self.__model)
			self.priority.setPrettyPrintMathFormula(priority)

		else:
			self.priority = priority


	def getPriority(self):

		if self.priority is not None:
			return self.priority.getPrettyPrintMathFormula()

	def getPriorityMath(self):

		return self.priority

	def addEventAssignment(self, variable=None, definition=None):

		t_assignment = EventAssignment(self.__model, len(self.listOfEventAssignments), self)
		self.listOfEventAssignments.append(t_assignment)
		if variable is not None and definition is not None:
			t_assignment.setVariable(variable)
			t_assignment.setPrettyPrintAssignment(definition)
		return t_assignment

	def renameSbmlId(self, old_sbml_id, new_sbml_id):

		self.trigger.renameSbmlId(old_sbml_id, new_sbml_id)

		for event_assignment in self.listOfEventAssignments:
			event_assignment.renameSbmlId(old_sbml_id, new_sbml_id)

		if self.delay is not None:
			self.delay.renameSbmlId(old_sbml_id, new_sbml_id)

		if self.priority is not None:
			self.priority.renameSbmlId(old_sbml_id, new_sbml_id)

	def memorySize(self):

		size = 0
		if self.useValuesFromTriggerTime:
			size += len(self.listOfEventAssignments)
		return size

	def isValid(self):
		return (
			self.trigger is not None and self.trigger.isValid()
			and len([ass for ass in self.listOfEventAssignments if ass.isValid()]) > 0
		)

	def getByXPath(self, xpath):

		if len(xpath) == 0:
			return self

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@value":
			return self.getValue()

		elif xpath[0] == "@name":
			return self.getName()

		elif xpath[0] == "@id":
			return self.getSbmlId()


	def getXPath(self, attribute=None):

		xpath = "sbml:event"
		if self.__model.sbmlLevel == 1:
			xpath += "[@name='%s']" % self.getSbmlId()
		else:
			xpath += "[@id='%s']" % self.getSbmlId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.Compartment import Compartment
from libsignetsim.model.ModelException import CannotDeleteException, InvalidXPath
from libsignetsim.settings.Settings import Settings

from re import match


class ListOfCompartments(ListOf, HasIds, SbmlObject, HasParentObj):
	""" Class for the listOfCompartments in a sbml model """

	def __init__ (self, model, parent_obj):

		self.__model = model

		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		SbmlObject.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)

	def readSbml(self, sbml_compartments,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads compartments' list from a sbml file """

		for compartment in sbml_compartments:
			t_compartment = Compartment(self.__model, self, self.nextId())
			t_compartment.readSbml(compartment, sbml_level, sbml_version)
			ListOf.add(self, t_compartment)

		SbmlObject.readSbml(self, sbml_compartments, sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes compartments' list to a sbml file """

		for compartment in self:
			compartment.writeSbml(sbml_model, sbml_level, sbml_version)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbml_model.getListOfCompartments(), sbml_level, sbml_version)


	def new(self, name=None, sbml_id=None, value=1, constant=True, unit=None):
		""" Creates a new compartment """

		t_compartment = Compartment(self.__model, self, self.nextId())
		t_compartment.new(name, sbml_id, value, constant, unit)
		ListOf.add(self, t_compartment)
		return t_compartment

	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factors=None):

		if obj not in deletions:

			SbmlObject.copy(self, obj)

			for compartment in obj:
				if compartment not in deletions:

					t_compartment = Compartment(self.__model, self, self.nextId())
					t_compartment.copy(
						compartment,
						sids_subs=sids_subs,
						symbols_subs=symbols_subs,
						usids_subs=usids_subs,
						conversion_factors=conversion_factors
					)
					ListOf.add(self, t_compartment)


	def remove(self, comp):
		""" Remove an object from the list """

		t_nb_species = comp.getNbSpecies()
		if t_nb_species > 0:
			message = "Compartment contains %d species" % t_nb_species
			raise CannotDeleteException(message)

		self.__model.listOfVariables.removeVariable(comp)
		ListOf.remove(self, comp)


	def removeById(self, obj_id):
		""" Remove an object from the list """
		self.remove(ListOf.getById(self, obj_id))


	def resolveXPath(self, selector):

		if not (selector.startswith("compartment") or selector.startswith("sbml:compartment")):
			raise InvalidXPath(selector)

		res_match = match(r'(.*)\[@(.*)=(.*)\]', selector)
		if res_match is None:
			raise InvalidXPath(selector)

		tokens = res_match.groups()
		if len(tokens) != 3:
			raise InvalidXPath(selector)

		object = None
		if tokens[1] == "id":
			object = self.getBySbmlId(tokens[2][1:-1])
		elif tokens[1] == "name":
			object = self.getByName(tokens[2][1:-1])
		elif tokens[1] == "metaid":
			object = self.getByMetaId(tokens[2][1:-1])

		if object is not None:
			return object

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)

	def getXPath(self):
		return "/".join([self.getParentObj().getXPath(), "sbml:listOfCompartments"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


class InitiallyAssignedVariable(object):

	def __init__(self, model):

		self.__model = model

		self.__hasInitialAssignment = False
		self.__hasInitialAssignmentBy = None


	# Modifiers
	def setInitialAssignmentBy(self, initial_assignment, shift=0):
		self.__hasInitialAssignment = True
		self.__hasInitialAssignmentBy = initial_assignment.objId + shift


	def unsetInitialAssignmentBy(self):
		self.__hasInitialAssignment = False
		self.__hasInitialAssignmentBy = None


	# Queries
	def hasInitialAssignment(self):
		return self.__hasInitialAssignment

	def hasInitialAssignmentBy(self):
		if self.hasInitialAssignment():
			return self.__model.listOfInitialAssignments[self.__hasInitialAssignmentBy]
		else:
			return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the research for conservation laws, and the subsequent reduction

"""

from libsignetsim import Model, MathFormula, KineticLaw

from libsignetsim.model.math.sympy_shortcuts import SympyEqual, SympyZero
from unittest import TestCase
from sympy import simplify


class TestMath(TestCase):
	""" Tests high level functions """


	def testPrettyPrint(self):

		model = Model()

		c = model.listOfCompartments.new("c")

		s1 = model.listOfSpecies.new("s1")
		s2 = model.listOfSpecies.new("s2")
		s3 = model.listOfSpecies.new("s3")

		s4 = model.listOfSpecies.new("s4", hasOnlySubstanceUnits=True)
		s5 = model.listOfSpecies.new("s5", hasOnlySubstanceUnits=True)
		s6 = model.listOfSpecies.new("s6", hasOnlySubstanceUnits=True)


		f1 = MathFormula(model)
		f1.setPrettyPrintMathFormula("s1+s2*s3")

		f2 = MathFormula(model)
		f2.setPrettyPrintMathFormula("s1/c + (s2/c)*(s3/c)", rawFormula=True)

		f3 = MathFormula(model)
		f3.setPrettyPrintMathFormula("s4+s5*s6")

		f4 = MathFormula(model)
		f4.setPrettyPrintMathFormula("s4+s5*s6", rawFormula=True)

		self.assertTrue(self.sympyEqual(
			f1.getDeveloppedInternalMathFormula(),
			f2.getDeveloppedInternalMathFormula()
		))

		self.assertTrue(self.sympyEqual(
			f3.getDeveloppedInternalMathFormula(),
			f4.getDeveloppedInternalMathFormula()
		))

	def sympyEqual(self, a, b):

		# print "test %s == %s : %s" % (simplify(a), simplify(b), (simplify(a-b) == 0))
		# print "test %s == %s : %s" % (a, b, (simplify(a-b) == 0))
		return simplify(a-b) == 0


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.AbstractTask import AbstractTask
from libsignetsim.sedml.container.ListOfSetValueChanges import ListOfSetValueChanges
from libsignetsim.sedml.container.ListOfRanges import ListOfRanges
from libsignetsim.sedml.container.ListOfSubTasks import ListOfSubTasks
from libsignetsim.sedml.SedmlException import SedmlMixedSubtasks, SedmlMultipleModels
from libsignetsim.simulation.SteadyStatesSimulation import SteadyStatesSimulation
from libsignetsim.simulation.TimeseriesSimulation import TimeseriesSimulation
from libsignetsim.settings.Settings import Settings

class RepeatedTask(AbstractTask):

	def __init__(self, document):

		AbstractTask.__init__(self, document)

		self.__document = document
		self.__range = None
		self.__resetModel = None
		self.listOfSetValueChanges = ListOfSetValueChanges(self.__document, self)
		self.listOfRanges = ListOfRanges(self.__document, self)
		self.listOfSubTasks = ListOfSubTasks(self.__document)

		self.__simulationObject = None
		self.__times = None
		self.__results = None

	def readSedml(self, task, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		AbstractTask.readSedml(self, task, level, version)

		if task.isSetRangeId():
			self.__range = task.getRangeId()

		if task.isSetResetModel():
			self.__resetModel = task.getResetModel()

		self.listOfSetValueChanges.readSedml(task.getListOfTaskChanges(), level, version)
		self.listOfRanges.readSedml(task.getListOfRanges(), level, version)
		self.listOfSubTasks.readSedml(task.getListOfSubTasks(), level, version)

	def writeSedml(self, task, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		AbstractTask.writeSedml(self, task, level, version)

		if self.__range is not None:
			task.setRangeId(self.__range)

		if self.__resetModel is not None:
			task.setResetModel(self.__resetModel)

		self.listOfSetValueChanges.writeSedml(task.getListOfTaskChanges(), level, version)
		self.listOfRanges.writeSedml(task.getListOfRanges(), level, version)
		self.listOfSubTasks.writeSedml(task.getListOfSubTasks(), level, version)

	def setRangeReference(self, range_reference):
		self.__range = range_reference

	def setRange(self, range_obj):
		self.__range = range_obj.getId()

	def setResetModel(self, reset_model):
		self.__resetModel = reset_model


	def getModel(self):
		models = self.listOfSubTasks.getModels()
		if len(models) == 1:
			return models[0]
		else:
			raise SedmlMultipleModels("Multiple models are not implemented")

	def run(self, timeout=None):

		if not self.listOfSubTasks.hasSingleTypeOfTask():
			raise SedmlMixedSubtasks("Mixed subtasks are not implemented")

		if self.listOfSubTasks.hasOneSteps():
			models = self.listOfSubTasks.getModels()
			abs_tol = self.listOfSubTasks.getAbsTols()[0]
			rel_tol = self.listOfSubTasks.getRelTols()[0]
			range = self.listOfRanges.getByRangeId(self.__range)
			timepoints = range.getValuesArray()

			self.__simulationObject = TimeseriesSimulation(
				list_of_models=models,
				list_samples=timepoints,
				abs_tol=abs_tol,
				rel_tol=rel_tol,
				keep_files=True
			)
			self.__simulationObject.run(timeout=timeout)

			self.__times = self.__simulationObject.getRawData()[0][0]
			self.__results = self.__simulationObject.getRawData()[0][1]

		elif self.listOfSubTasks.hasSteadyStates():
			models = self.listOfSubTasks.getModels()
			value_changes = self.listOfSetValueChanges.getValueChanges()

			self.__simulationObject = SteadyStatesSimulation(
				list_of_models=models,
				species_input=list(value_changes.keys())[0],
				list_of_initial_values=list(value_changes.values())[0]
			)
			self.__simulationObject.run(timeout=timeout)
			self.__results = self.__simulationObject.getRawData()

		elif self.listOfSubTasks.hasUniformTimeCourses():
			pass


	def getResultsByVariable(self, variable_sbmlid):
		return self.__results[variable_sbmlid]

	def getTimes(self):
		return self.__times

	def getDuration(self):
		return self.__simulationObject.getSimulationDuration()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.container.ListOf import ListOf
from libsignetsim.numl.OntologyTerm import OntologyTerm
from libsignetsim.settings.Settings import Settings

class ListOfOntologyTerms(ListOf, list):

	def __init__(self, document):

		ListOf.__init__(self, document)
		self.__document = document
		self.__termCounter = 0

	def createOntologyTerm(self):
		ontology_term = OntologyTerm(self.__document)
		ontology_term.setId("term_%d" % self.__termCounter)
		ListOf.append(self, ontology_term)
		self.__termCounter += 1
		return ontology_term

	def readNuML(self, list_of_ontology_terms, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		ListOf.readNuML(self, list_of_ontology_terms, level, version)
		for ontology_term in list_of_ontology_terms:
			t_ontology_term = OntologyTerm(self.__document)
			t_ontology_term.readNuML(ontology_term, level, version)
			ListOf.append(self, t_ontology_term)

	def writeNuML(self, numl_document, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		# ListOf.writeNuML(self, numl_document., level, version)

		for ontology_term in self:
			t_ontology_term = numl_document.createOntologyTerm()
			ontology_term.writeNuML(t_ontology_term, level, version)

	def getByOntologyTerm(self, lookup_term):
		for ontology_term in self:
			if ontology_term.getId() == lookup_term:
				return ontology_term


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.Output import Output
from libsignetsim.sedml.container.ListOfCurves import ListOfCurves
from libsignetsim.figure.SigNetSimFigure import SigNetSimFigure
from libsignetsim.settings.Settings import Settings


class Plot2D(Output):

	def __init__(self, document):

		Output.__init__(self, document)

		self.__document = document
		self.listOfCurves = ListOfCurves(self.__document, self)

	def readSedml(self, plot, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		Output.readSedml(self, plot, level, version)
		self.listOfCurves.readSedml(plot.getListOfCurves(), level, version)

	def writeSedml(self, plot, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		Output.writeSedml(self, plot, level, version)
		self.listOfCurves.writeSedml(plot.getListOfCurves(), level, version)

	def showFigure(self):
		fig = SigNetSimFigure()
		subplot = fig.add_subplot(1,1,1)

		print_ynames = False
		if self.listOfCurves.getXAxisTitle() is not None:
			subplot.set_xlabel(self.listOfCurves.getXAxisTitle())

		if self.listOfCurves.getYAxisTitle() is not None:
			subplot.set_ylabel(self.listOfCurves.getYAxisTitle())
		else:
			print_ynames = True

		for i, curve in enumerate(self.listOfCurves):
			curve.build(fig, subplot, i, print_ynames)

		if print_ynames:
			subplot.legend(loc='upper right')

		subplot.set_title(self.getName())


	def getDataToGenerate(self):

		return self.listOfCurves.getDataToGenerate()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings


class CWriterModels(object):

	def __init__ (self, list_of_models,
				  	time_min,
					list_samples,
					abs_tol,
					rel_tol):

		self.listOfModels = list_of_models

		# Simulation variables
		self.timeMin = time_min
		self.listSamples = list_samples
		self.absTol = abs_tol
		self.relTol = rel_tol

	def writeModelsHeaders(self, f_h, f_c):

		# Writing headers for the c file
		f_c.write("/*****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *   model.c                                                     *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *   written by Vincent Noel                                     *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *   Model definition for simulation and optimization            *\n")
		f_c.write(" *   Generated by libSigNetSim                                      *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" *****************************************************************\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" * Copyright (C) 2012-2014 Vincent Noel                          *\n")
		f_c.write(" * the full GPL copyright notice can be found in lsa.c           *\n")
		f_c.write(" *                                                               *\n")
		f_c.write(" ****************************************************************/\n\n")
		f_c.write("#include <stdlib.h>\n")
		f_c.write("#include <stdio.h>\n")
		f_c.write("#include \"model.h\"\n\n")
		f_c.write("#include \"integrate/realtype_math.h\"\n\n")

		f_c.write("#define min(x, y) (((x) < (y)) ? (x) : (y))\n")
		f_c.write("#define max(x, y) (((x) > (y)) ? (x) : (y))\n")
		f_c.write("#define Ith(v,i)    NV_Ith_S(v,i-1)       /* Ith numbers components 1..NEQ */\n")
		f_c.write("#define IJth(A,i,j) DENSE_ELEM(A,i-1,j-1) /* IJth numbers rows,cols 1..NEQ */\n\n")

		# Writing headers for the h file
		f_h.write("/*****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *   model.h                                                     *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *   written by Vincent Noel                                     *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *   Model definition for simulation and optimization            *\n")
		f_h.write(" *   Generated by libSigNetSim                                      *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" *****************************************************************\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" * Copyright (C) 2012-2014 Vincent Noel                          *\n")
		f_h.write(" * the full GPL copyright notice can be found in lsa.c           *\n")
		f_h.write(" *                                                               *\n")
		f_h.write(" ****************************************************************/\n\n")

		f_h.write("#include \"integrate/models.h\"\n")
		f_h.write("#include \"integrate/integrate.h\"\n")

	def writeModelsInitialization(self, f_h, f_c):

		f_c.write("list_of_models models;\n\n")

		for modelInd, model in enumerate(self.listOfModels):
			f_c.write("ModelDefinition     model_%d;\n\n" % modelInd)

		f_c.write("list_of_models * getListOfModels()\n")
		f_c.write("{\n")
		f_c.write("  return &models;\n")
		f_c.write("}\n")

		f_h.write("list_of_models * getListOfModels();\n")
		f_h.write("double * getInitValues(int i);\n")
		f_c.write("\ndouble * getInitValues(int i)\n")
		f_c.write("{\n")
		f_c.write("  return NULL;\n")
		f_c.write("}\n")


		f_h.write("void init_models();\n")
		f_c.write("void init_models()\n{\n")

		f_c.write("  models.nb_models = " + str(len(self.listOfModels)) + ";\n")
		f_c.write("  models.models = malloc(sizeof(ModelDefinition)*models.nb_models);\n")

		for modelInd, _ in enumerate(self.listOfModels):
			f_c.write("  init_model_%d();\n" % modelInd)
			f_c.write("  models.models[%d] = &model_%d;\n" % (modelInd, modelInd))

		f_c.write("}\n\n")


	def writeModelsFinalization(self, f_h, f_c):

		f_h.write("void finalize_models();\n")
		f_c.write("void finalize_models()\n{\n")

		for modelInd, _ in enumerate(self.listOfModels):
			f_c.write("  finalize_model_%d();\n" % modelInd)

		f_c.write("  free(models.models);\n")

		f_c.write("}\n\n")


	def writeModelFiles(self):

		f_c = open(self.getTempDirectory() + Settings.C_generatedDirectory_v2 + "model.c", 'w')
		f_h = open(self.getTempDirectory() + Settings.C_generatedDirectory_v2 + "model.h", 'w')

		self.writeModelsHeaders(f_h, f_c)
		self.writeModelsInitialization(f_h, f_c)
		self.writeModelsFinalization(f_h, f_c)

		for i_model, model in enumerate(self.listOfModels):
			model.writeCCode(f_h, f_c, i_model, self.timeMin[i_model], self.listSamples[i_model], self.absTol[i_model], self.relTol[i_model])

		f_c.close()
		f_h.close()
